/*------------------------------------------------------------------------------
* Alpha Shooter
* A 3D OpenGL First-Person Shooter game
* Copyright (C) 2005, 2006, 2007 Nicola Cocchiaro <ncocchiaro@users.sf.net>
* Alpha Shooter can be found at http://www.sf.net/projects/alphashooter.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 or, (at your option) any later
* version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*-------------------------------------------------------------------------------
*/


// controls.c
// keyboard and mouse handlers for glut functions

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "controls.h"

/* Turns the six lights off */
void lights_off()
{
	glDisable(GL_LIGHT2);
 	glDisable(GL_LIGHT3);
	glDisable(GL_LIGHT4);
	glDisable(GL_LIGHT5);
	glDisable(GL_LIGHT6);
	glDisable(GL_LIGHT7);
}

/* Turns the six lights on */
void lights_on()
{
	glEnable(GL_LIGHT2);
 	glEnable(GL_LIGHT3);
	glEnable(GL_LIGHT4);
	glEnable(GL_LIGHT5);
	glEnable(GL_LIGHT6);
	glEnable(GL_LIGHT7);
}

/* Pauses and unpauses the game */
void pause()
{
	if (state->pause)
	{
		if (debug)
			strcpy(state->status_string, "Debug mode on");
		else
			strcpy(state->status_string, "");
		state->pause = 0;
		glutAttachMenu(GLUT_RIGHT_BUTTON);
		glutIdleFunc(idle);
	}
	else
	{
		strcpy(state->status_string, "Game paused");
		state->pause = 1;
		glutDetachMenu(GLUT_RIGHT_BUTTON);
		glutPostRedisplay();
		glutIdleFunc(NULL);
	}
}

/* Menu status callback function */
void menuStatus(int status, int x, int y)
{
	if (status == GLUT_MENU_IN_USE)
	{
		strcpy(state->status_string, "Game paused");
		state->pause = 1;
		glutPostRedisplay();
		glutIdleFunc(NULL);
	}
	else
	{
		if (debug)
			strcpy(state->status_string, "Debug mode on");
		else	
			strcpy(state->status_string, "");
		state->pause = 0;
		glutIdleFunc(idle);
	}
}

/* Menu select callback function */
void menuSelect(int value)
{
	switch(value)
	{
		case 'O':
			state->lines = !state->lines;
			break;
		case 'T':
			state->trajectories = !state->trajectories;
			break;
		case 'x':
			state->cannons_linked = !state->cannons_linked;
			break;
		case 'm':
			state->mouse_mode = (state->mouse_mode + 1)
				% MOUSE_MODES;
			state->mouse_posx = 0;
			state->mouse_posy = 0;
			switch(state->mouse_mode)
			{
				case MOUSE_NONE:
					fprintf(stdout, "No mouse movement\n");
					break;
				case MOUSE_PROGRESSIVE:
					state->mouse_posx = state->windowx/2;
					state->mouse_posy = state->windowy/2;
					glutWarpPointer(state->windowx/2, state->windowy/2);
					fprintf(stdout, "Progressive mouse movement\n");
					break;
				case MOUSE_PERSISTENT:
					fprintf(stdout, "Persistent mouse movement\n");
					break;
				default:
					break;
			}
			break;
		case 'j':
			if (state->jetpack)
				cam->leveling = 1;
			state->jetpack = !state->jetpack;
			break;
		case 'p':
			pause();
			break;
		case 'l':
			if (state->light1)
			{
				glDisable(GL_LIGHT1);
				fprintf(stdout, "Headlight off\n");
			}
			else
			{
				glEnable(GL_LIGHT1);
				fprintf(stdout, "Headlight on\n");
			}
			state->light1 = !state->light1;
			break;
		case 'L':
			if (state->light0 == 1)
			{
				lights_off();
				state->light0 = 0;
				fprintf(stdout, "Main lights off\n");
			}
			else if (state->light0 == 0)
			{
				state->light0 = 1;
				fprintf(stdout, "Main lights on\n");
				lights_on();
			}
			break;
		/* don't need this at the moment *
		case 'k':
			if (glIsEnabled(GL_FOG) == GL_FALSE)
				glEnable(GL_FOG);
			else
				glDisable(GL_FOG);
			break;
		*/
		case 'f':
			if (state->fullscreen)
			{
				glutReshapeWindow(res[state->curr_res][0], res[state->curr_res][1]);
				fprintf(stdout, "Set resolution to %d x %d\n", res[state->curr_res][0], res[state->curr_res][1]);
			}
			else
			{
				glutFullScreen();
				fprintf(stdout, "Fullscreen mode\n");
			}
			state->fullscreen = !state->fullscreen;
			break;
		case 'g':
			state->curr_res = (state->curr_res + 1) % MAX_RES;
			state->windowx = res[state->curr_res][0];
			state->windowy = res[state->curr_res][1];
			state->fullscreen = 0;
			glutReshapeWindow(state->windowx, state->windowy);
			fprintf(stdout, "Set resolution to %d x %d\n", res[state->curr_res][0], res[state->curr_res][1]);
			break;
		case 27:
			exit(0);
			break;
		default:
			break;
	}
}

/* Switches from first to third person and viceversa */
/* Unimplemented */
void switch_person()
{
	/* switch viewpoint */
	//state->third_person = !state->third_person;
}

/* Shoots cannons */
void shoot()
{
	GLint cannon_state = state->left_cannon_fired;
	
	if (state->pause)
		return;
	if (((cannon_state == 0) || (state->cannons_linked)) 
		&& (left_shot->t == 0))
	{
		left_shot->orientation = cam->orientation;
//		left_shot->t = 0;
		left_shot->radius = 0.1;
//		left_shot->rotate_around_x = cam->total_rot_x;
//		left_shot->rotate_around_y = cam->total_rot_y;
//		left_shot->rotate_around_z = cam->total_rot_z;
		copy_vector(cam->old_pos, &(left_shot->vp));
		copy_vector(cam->old_pos, &(left_shot->init_vp));
		copy_vector(cam->old_pos, &(left_shot->final_vp));
		translate(0.5, -0.5, 1.0, &(left_shot->init_vp));
		translate(0.4, 0.0, 31.0, &(left_shot->final_vp));
		state->left_cannon_fired = 1;
		left_shot->live = 1;
		new_trail(1, left_shot->vp);
	}
	if (((cannon_state == 1) || (state->cannons_linked))
		&& (right_shot->t == 0))
	{
		right_shot->orientation = cam->orientation;
//		right_shot->t = 0;
		right_shot->radius = 0.1;
//		right_shot->rotate_around_x = cam->total_rot_x;
//		right_shot->rotate_around_y = cam->total_rot_y;
//		right_shot->rotate_around_z = cam->total_rot_z;
		copy_vector(cam->old_pos, &(right_shot->vp));
		copy_vector(cam->old_pos, &(right_shot->init_vp));
		copy_vector(cam->old_pos, &(right_shot->final_vp));
		translate(-0.5, -0.5, 1.0, &(right_shot->init_vp));
		translate(-0.4, 0.0, 31.0, &(right_shot->final_vp));
		state->left_cannon_fired = 0;
		right_shot->live = 1;
		new_trail(1, right_shot->vp);
	}
}

/* Activates an object if close enough */
void activate_object()
{
	vector console_pos = { 4.5, 1.8, 10.0, 1.0 };

	if (distance(cam->pos, console_pos) < (cam->radius+2.0))
	{
		if (state->shooter == 0)
		{
			state->light0 = 0;
			lights_off();
			state->shooter = 1;
		}
		else
		{
			state->light0 = 1;
			lights_on();
			state->shooter = 0;
			target->live = 0;
			target->t = 0.0;
		}
	}
}

/* Handles ASCII keys */
void keyboard_handler(unsigned char key, int x, int y)
{
	char done = 0;
	
	/* keys that are always checked */
	switch(key)
	{
		case 'F':
			if (state->fullscreen)
			{
				glutReshapeWindow(res[state->curr_res][0], res[state->curr_res][1]);
				fprintf(stdout, "Set resolution to %d x %d\n", res[state->curr_res][0], res[state->curr_res][1]);
			}
			else
			{
				glutFullScreen();
				fprintf(stdout, "Fullscreen mode\n");
			}
			state->fullscreen = !state->fullscreen;
			done = 1;
			break;
		case 'G':
			state->curr_res = (state->curr_res + 1) % MAX_RES;
			state->windowx = res[state->curr_res][0];
			state->windowy = res[state->curr_res][1];
			state->fullscreen = 0;
			glutReshapeWindow(state->windowx, state->windowy);
			fprintf(stdout, "Set resolution to %d x %d\n", res[state->curr_res][0], res[state->curr_res][1]);
			done = 1;
			break;
		/* ESC key */
		case 27:
			fprintf(stdout, "Quit\n");
			exit(0);
			break;
		case 'p':
			pause();
			done = 1;
			break;
		default:
			break;
	}
	if ((!state->pause) && (!done))
	{
		/* keys that are checked only when the game is not paused */
		switch(key)
		{
			case 'q':
				if (state->jetpack)
					state->keypress[KEY_ROLL_LT] = 1;
				else
					state->keypress[KEY_ROT_LT] = 1;
				break;
			case 'w':
				state->keypress[KEY_FW] = 1;
				break;
			case 'e':
				if (state->jetpack)
					state->keypress[KEY_ROLL_RT] = 1;
				else
					state->keypress[KEY_ROT_RT] = 1;
				break;
			case 'r':
				if (state->jetpack)
					state->keypress[KEY_UP] = 1;
				break;
			case 'a':
				state->keypress[KEY_LEFT] = 1;
				break;
			case 's':
				state->keypress[KEY_BACK] = 1;
				break;
			case 'd':
				state->keypress[KEY_RIGHT] = 1;
				break;
			case 'f':
				if (state->jetpack)
					state->keypress[KEY_DOWN] = 1;
				break;
			case 'x':
				state->cannons_linked = !state->cannons_linked;
				break;
			/* reset (debug) */
			case 'R':
				if (debug)
					reset_view();
				break;
			case 'P':
				//switch_person();
				break;
			case 'j':
				if (state->jetpack)
					cam->leveling = 1;
				state->jetpack = !state->jetpack;
				break;
			case 'l':
				if (state->light1)
				{
					glDisable(GL_LIGHT1);
					fprintf(stdout, "Headlight off\n");
				}
				else
				{
					glEnable(GL_LIGHT1);
					fprintf(stdout, "Headlight on\n");
				}
				state->light1 = !state->light1;
				break;
			case 'L':
				if (state->light0 == 1)
				{
					lights_off();
					state->light0 = 0;
					fprintf(stdout, "Main lights off\n");
				}
				else if (state->light0 == 0)
				{
					state->light0 = 1;
					fprintf(stdout, "Main lights on\n");
					lights_on();
				}
				break;
			case 'm':
				state->mouse_mode = (state->mouse_mode + 1)
					% MOUSE_MODES;
				state->mouse_posx = 0;
				state->mouse_posy = 0;
				switch(state->mouse_mode)
				{
				case MOUSE_NONE:
					fprintf(stdout, "No mouse movement\n");
					break;
				case MOUSE_PROGRESSIVE:
					state->mouse_posx = state->windowx/2;
					state->mouse_posy = state->windowy/2;
					glutWarpPointer(state->windowx/2, state->windowy/2);
					fprintf(stdout, "Progressive mouse movement\n");
					break;
				case MOUSE_PERSISTENT:
					fprintf(stdout, "Persistent mouse movement\n");
					break;
				default:
					break;
				}
				break;
			case 'g':
				activate_object();
				break;
			case 'k':
				if (debug)
				{
					if (glIsEnabled(GL_FOG) == GL_FALSE)
						glEnable(GL_FOG);
					else
						glDisable(GL_FOG);
				}
				break;
			case ' ':
				shoot();
				break;
			case 9:
				cam->leveling = !cam->leveling;
				break;
			default:
				break;
		}
	}
}

/* Handles key released events */
void release_key_handler(unsigned char key, int x, int y)
{
	switch (key)
	{
		case 'w':
			state->keypress[KEY_FW] = 0;
			break;
		case 'a':
			state->keypress[KEY_LEFT] = 0;
			break;
		case 's':
			state->keypress[KEY_BACK] = 0;
			break;
		case 'd':
			state->keypress[KEY_RIGHT] = 0;
			break;
		case 'q':
			if (state->jetpack)
				state->keypress[KEY_ROLL_LT] = 0;
			else
				state->keypress[KEY_ROT_LT] = 0;
			break;
		case 'e':
			if (state->jetpack)
				state->keypress[KEY_ROLL_RT] = 0;
			else
				state->keypress[KEY_ROT_RT] = 0;
			break;
		case 'r':
			if (state->jetpack)
				state->keypress[KEY_UP] = 0;
			break;
		case 'f':
			if (state->jetpack)
				state->keypress[KEY_DOWN] = 0;
			break;
		default:
			break;
	}
}

/* Handles special key release events */
void special_release_handler(int key, int x, int y)
{
	switch(key)
	{
		case GLUT_KEY_UP:
			state->specialkey[SKEY_ROT_UP] = 0;
			break;
		case GLUT_KEY_DOWN:
			state->specialkey[SKEY_ROT_DN] = 0;
			break;
		case GLUT_KEY_LEFT:
			state->specialkey[SKEY_ROT_LT] = 0;
			break;
		case GLUT_KEY_RIGHT:
			state->specialkey[SKEY_ROT_RT] = 0;
			break;
		/* keypad 7 */
		case 104:
			if (state->jetpack)
				state->specialkey[SKEY_ROLL_LT] = 0;
			break;
		/* keypad 9 */
		case 106:
			if (state->jetpack)
				state->specialkey[SKEY_ROLL_RT] = 0;
			break;
		default:
			break;
	}
}

/* Handles special keys */
void special_handler(int key, int x, int y)
{
	if (state->pause)
		return;
	switch(key)
	{
		case GLUT_KEY_UP:
			state->specialkey[SKEY_ROT_UP] = 1;
			break;
		case GLUT_KEY_DOWN:
			state->specialkey[SKEY_ROT_DN] = 1;
			break;
		case GLUT_KEY_LEFT:
			state->specialkey[SKEY_ROT_LT] = 1;
			break;
		case GLUT_KEY_RIGHT:
			state->specialkey[SKEY_ROT_RT] = 1;
			break;
		/* keypad 7 */
		case 104:
			if (state->jetpack)
				state->specialkey[SKEY_ROLL_LT] = 1;
			break;
		/* keypad 9 */
		case 106:
			if (state->jetpack)
				state->specialkey[SKEY_ROLL_RT] = 1;
			break;
		default:
			break;
	}
}

/* Checks for movement and updates camera state accordingly */
void check_for_movement()
{
	if (state->keypress[KEY_FW] == 1)
		cam->to_shift_z += fps->shift;
	if (state->keypress[KEY_BACK] == 1)
		cam->to_shift_z -= fps->shift;
	if (state->keypress[KEY_LEFT] == 1)
		cam->to_shift_x += fps->shift;
	if (state->keypress[KEY_RIGHT] == 1)
		cam->to_shift_x -= fps->shift;
	if (state->keypress[KEY_UP] == 1)
		cam->to_shift_y += fps->shift;
	if (state->keypress[KEY_DOWN] == 1)
		cam->to_shift_y -= fps->shift;
	if (state->keypress[KEY_ROLL_LT] == 1)
		rotate_around_Z(fps->rot, cam);
	if (state->keypress[KEY_ROLL_RT] == 1)
		rotate_around_Z(-fps->rot, cam);
	if (state->keypress[KEY_ROT_LT] == 1)
		rotate_around_Y(-fps->rot, cam);
	if (state->keypress[KEY_ROT_RT] == 1)
		rotate_around_Y(fps->rot, cam);
	if (state->specialkey[SKEY_ROT_UP] == 1)
		rotate_around_X(fps->rot, cam);
	if (state->specialkey[SKEY_ROT_DN] == 1)
		rotate_around_X(-fps->rot, cam);
	if (state->specialkey[SKEY_ROT_LT] == 1)
		rotate_around_Y(-fps->rot, cam);
	if (state->specialkey[SKEY_ROT_RT] == 1)
		rotate_around_Y(fps->rot, cam);
	if (state->specialkey[SKEY_ROLL_LT] == 1)
		rotate_around_Z(-fps->rot, cam);
	if (state->specialkey[SKEY_ROLL_RT] == 1)
		rotate_around_Z(fps->rot, cam);
}

/* Handles mouse button presses */
void mouse_handler(int button, int s, int x, int y)
{
	if (state->pause)
		return;
	if ((button == GLUT_MIDDLE_BUTTON) && (s == GLUT_UP))
		shoot();
}

/* Handles mouse drags */
//void motion_handler(int x, int y)
//{
//
//}

/* Handles mouse motion */
void passive_handler(int x, int y)
{
	/* dead zone */
	GLint left_bound   = state->windowx/2 - 50;
	GLint right_bound  = state->windowx/2 + 50;
	GLint top_bound    = state->windowy/2 - 50;
	GLint bottom_bound = state->windowy/2 + 50;
	GLdouble scale_factor = 0.5;
	GLdouble scale_prog = 15;

	/* no mouse view or game paused, do nothing */
	if ((state->mouse_mode == MOUSE_NONE) || (state->pause))
		return;

	/* PERSISTENT mode, view rotates according to mouse cursor position
	 * on screen, even if mouse is not moving */
	if (state->mouse_mode == MOUSE_PERSISTENT)
	{
		state->mouse_posx = x;
		state->mouse_posy = y;

		if ((x > left_bound) && (x < right_bound) &&
				(y > top_bound ) && (y < bottom_bound))
		{
			state->xprop = 0.0;
			state->yprop = 0.0;
			return;
		}

		if (x > right_bound)
			state->yprop = (GLdouble)(x-right_bound)/
			(GLdouble)(state->windowx-right_bound)/scale_factor;
		else if (x < left_bound)
			state->yprop = -1*(1 - (GLdouble)(x)/
			(GLdouble)left_bound)/scale_factor;

		if (y < top_bound)
			state->xprop = (1 - (GLdouble)(y)/
			(GLdouble)top_bound)/scale_factor;
		else if (y > bottom_bound)
			state->xprop = -1*(GLdouble)(y-bottom_bound)/
			(GLdouble)(state->windowy-bottom_bound)/scale_factor;
	}

	/* PROGRESSIVE mode, view rotates as mouse is moved on screen */	
	else if (state->mouse_mode == MOUSE_PROGRESSIVE)
	{
		state->window_xpos = glutGet(GLUT_WINDOW_X);
		state->window_ypos = glutGet(GLUT_WINDOW_Y);
		
		if (x != state->mouse_posx)
		{
			state->prev_xdiff = abs(x-state->mouse_posx);
			if (x < state->mouse_posx)
				rotate_around_Y((-fps->rot*state->prev_xdiff)/scale_prog, cam);
			else
				rotate_around_Y((fps->rot*state->prev_xdiff)/scale_prog, cam);
		}
		else 
		{
			if (state->prev_xdiff < 40)
				state->prev_xdiff = 40;
			if (x <= state->window_xpos)
				rotate_around_Y((-fps->rot*state->prev_xdiff)/scale_prog, cam);
			else if (x >= state->window_xpos+state->windowx-1)
				rotate_around_Y((fps->rot*state->prev_xdiff)/scale_prog, cam);
		}

		if (y != state->mouse_posy)
		{
			state->prev_ydiff = abs(y-state->mouse_posy);
			if (y < state->mouse_posy)
				rotate_around_X((fps->rot*state->prev_ydiff)/scale_prog, cam);
			else
				rotate_around_X((-fps->rot*state->prev_ydiff)/scale_prog, cam);
		}
		else
		{
			if (state->prev_ydiff < 40)
				state->prev_ydiff = 40;
			if (y <= state->window_ypos)
				rotate_around_X((fps->rot*state->prev_ydiff)/scale_prog, cam);
			else if (y >= state->window_ypos+state->windowy-1)	
				rotate_around_X((-fps->rot*state->prev_ydiff)/scale_prog, cam);
		}
		
		state->mouse_posx = x;
		state->mouse_posy = y;
	}
}
