/*------------------------------------------------------------------------------
* Alpha Shooter
* A 3D OpenGL First-Person Shooter game
* Copyright (C) 2005, 2006, 2007 Nicola Cocchiaro <ncocchiaro@users.sf.net>
* Alpha Shooter can be found at http://www.sf.net/projects/alphashooter.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 or, (at your option) any later
* version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*-------------------------------------------------------------------------------
*/


// dlists.c
// functions to set display lists

#include <stdio.h>
#include <stdlib.h>
#include "dlists.h"
#include "lib.h"

/* Draws a square tile on the XZ plane */
void draw_xz_tile(GLfloat i, GLfloat k, GLfloat j)
{
	GLint status;
	
	glBegin(GL_QUADS);

	glGetIntegerv(GL_FRONT_FACE, &status);
	if (status == GL_CW)
		glNormal3f(0.0, 1.0, 0.0);
	else
		glNormal3f(0.0, -1.0, 0.0);

	if (state->hi_res)
	{
		glTexCoord2f(0.0, 0.0); glVertex3f(i, k, j-1.0);
		glTexCoord2f(0.0, 0.5); glVertex3f(i, k, j-0.5);
		glTexCoord2f(0.5, 0.5); glVertex3f(i-0.5, k, j-0.5);
		glTexCoord2f(0.5, 0.0); glVertex3f(i-0.5, k, j-1.0);
		
		glTexCoord2f(0.0, 0.5); glVertex3f(i, k, j-0.5);
		glTexCoord2f(0.0, 1.0); glVertex3f(i, k, j);
		glTexCoord2f(0.5, 1.0); glVertex3f(i-0.5, k, j);
		glTexCoord2f(0.5, 0.5); glVertex3f(i-0.5, k, j-0.5);

		glTexCoord2f(0.5, 0.5); glVertex3f(i-0.5, k, j-0.5);
		glTexCoord2f(0.5, 1.0); glVertex3f(i-0.5, k, j);
		glTexCoord2f(1.0, 1.0); glVertex3f(i-1.0, k, j);
		glTexCoord2f(1.0, 0.5); glVertex3f(i-1.0, k, j-0.5);
		
		glTexCoord2f(0.5, 0.0); glVertex3f(i-0.5, k, j-1.0);
		glTexCoord2f(0.5, 0.5); glVertex3f(i-0.5, k, j-0.5);
		glTexCoord2f(1.0, 0.5); glVertex3f(i-1.0, k, j-0.5);
		glTexCoord2f(1.0, 0.0); glVertex3f(i-1.0, k, j-1.0);
	}
	else
	{
		glTexCoord2f(0.0, 0.0); glVertex3f(i, k, j-1.0);
		glTexCoord2f(0.0, 1.0); glVertex3f(i, k, j);
		glTexCoord2f(1.0, 1.0); glVertex3f(i-1.0, k, j);
		glTexCoord2f(1.0, 0.0); glVertex3f(i-1.0, k, j-1.0);
	}
	
	glEnd();
}

/* Draws a square tile on the XY plane */
void draw_xy_tile(GLfloat i, GLfloat k, GLfloat j)
{
	GLint status;

	glBegin(GL_QUADS);
	
	glGetIntegerv(GL_FRONT_FACE, &status);
	if (status == GL_CW)
		glNormal3f(0.0, 0.0, -1.0);
	else
		glNormal3f(0.0, 0.0, 1.0);
	
	if (state->hi_res)
	{
		glTexCoord2f(0.0, 0.0); glVertex3f(i, k, j);
		glTexCoord2f(0.0, 0.5); glVertex3f(i, k+0.5, j);
		glTexCoord2f(0.5, 0.5); glVertex3f(i-0.5, k+0.5, j);
		glTexCoord2f(0.5, 0.0); glVertex3f(i-0.5, k, j);

		glTexCoord2f(0.0, 0.5); glVertex3f(i, k+0.5, j);
		glTexCoord2f(0.0, 1.0); glVertex3f(i, k+1.0, j);
		glTexCoord2f(0.5, 1.0); glVertex3f(i-0.5, k+1.0, j);
		glTexCoord2f(0.5, 0.5); glVertex3f(i-0.5, k+0.5, j);

		glTexCoord2f(0.5, 0.5); glVertex3f(i-0.5, k+0.5, j);
		glTexCoord2f(0.5, 1.0); glVertex3f(i-0.5, k+1.0, j);
		glTexCoord2f(1.0, 1.0); glVertex3f(i-1.0, k+1.0, j);	
		glTexCoord2f(1.0, 0.5); glVertex3f(i-1.0, k+0.5, j);

		glTexCoord2f(0.5, 0.0); glVertex3f(i-0.5, k, j);
		glTexCoord2f(0.5, 0.5); glVertex3f(i-0.5, k+0.5, j);
		glTexCoord2f(1.0, 0.5); glVertex3f(i-1.0, k+0.5, j);
		glTexCoord2f(1.0, 0.0); glVertex3f(i-1.0, k, j);
	}
	else
	{
		glTexCoord2f(0.0, 0.0); glVertex3f(i, k, j);
		glTexCoord2f(0.0, 1.0); glVertex3f(i, k+1.0, j);
		glTexCoord2f(1.0, 1.0); glVertex3f(i-1.0, k+1.0, j);
		glTexCoord2f(1.0, 0.0); glVertex3f(i-1.0, k, j);
	}
	
	glEnd();
}

/* Draws a monitor object */
void draw_monitor(GLuint texture)
{
	glPushMatrix();
	glEnable(GL_TEXTURE_2D);
	glCallList(dl->monitor_base_list);
	glBindTexture(GL_TEXTURE_2D, tex->tex_names[texture]);

	glBegin(GL_QUADS);

	glNormal3f(1.0, 0.0, 0.0);
	glTexCoord2f(0.0, 0.0); glVertex3f(0.05, -0.8, 1.25);
	glTexCoord2f(0.0, 1.0); glVertex3f(0.05, 0.8, 1.25);
	glTexCoord2f(1.0, 1.0); glVertex3f(0.05, 0.8, -1.25);
	glTexCoord2f(1.0, 0.0); glVertex3f(0.05, -0.8, -1.25);

	glEnd();

	glDisable(GL_TEXTURE_2D);
	glPopMatrix();
}

/* Draws a platform object */
void draw_platform(GLuint texture)
{
	GLfloat i, j;
	
	glPushMatrix();
	glEnable(GL_TEXTURE_2D);
	glCallList(dl->platform_list);
	glBindTexture(GL_TEXTURE_2D, tex->tex_names[texture]);

	for (i = 2.0; i > -2.0; i -= 1.0)
		for (j = 2.0; j > -2.0; j -= 1.0)
			draw_xz_tile(i, 0.1, j);

	glDisable(GL_TEXTURE_2D);
	glPopMatrix();
}

/* display list to set crosshair properties */
void set_crosshair_material()
{
	GLfloat emit[] = { 0.0, 1.0, 0.0, 1.0 };
	GLfloat amb_diff[] = { 0.0, 1.0, 0.0, 0.5 };
	
	dl->crosshair_list = glGenLists(1);
	if (dl->crosshair_list != 0)
	{
		glNewList(dl->crosshair_list, GL_COMPILE);
		glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, amb_diff);
		glMaterialfv(GL_FRONT, GL_EMISSION, emit);
		glEndList();
	}
	else
	{
		fprintf(stderr, "Unable to initialize crosshair display list, aborting\n");
		exit(1);
	}
}

/* display list to turn emit material property off */
void set_emit_off()
{
	GLfloat emit_off[] = { 0.0, 0.0, 0.0, 1.0 };

	dl->emit_off_list = glGenLists(1);
	if (dl->emit_off_list != 0)
	{
		glNewList(dl->emit_off_list, GL_COMPILE);
		glMaterialfv(GL_FRONT, GL_EMISSION, emit_off);
		glEndList();
	}
	else
	{
		fprintf(stderr, "Unable to initialize emit_off display list, aborting\n");
		exit(1);
	}
}

/* sets a neutral white color (useful for modulating textures) */
void set_white_list()
{
	GLfloat emit[] = { 0.0, 0.0, 0.0, 1.0 };
	GLfloat specular[] = { 0.1, 0.1, 0.1, 1.0 };
	GLfloat mat_diffuse[] = { 0.8, 0.8, 0.8, 1.0 };
	
	dl->white_list = glGenLists(1);
	if (dl->white_list != 0)
	{
		glNewList(dl->white_list, GL_COMPILE);
		glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, mat_diffuse);
		glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, emit);
		glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specular);
		glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, 50.0);
		glEndList();
	}
	else
	{
		fprintf(stderr, "Unable to initialize white display list, aborting\n");
		exit(1);
	}
}

/* sets a green emit color */
void set_green_list()
{
	GLfloat emit[] = { 0.0, 1.0, 0.0, 1.0 };
	GLfloat specular[] = { 0.0, 1.0, 0.0, 0.5 };
	GLfloat mat_diffuse[] = { 0.5, 1.0, 0.5, 0.5 };
	
	dl->green_list = glGenLists(1);
	if (dl->green_list != 0)
	{
		glNewList(dl->green_list, GL_COMPILE);
		glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, mat_diffuse);
		glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, emit);
		glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specular);
		glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, 50.0);
		glEndList();
	}
	else
	{
		fprintf(stderr, "Unable to initialize green display list, aborting\n");
		exit(1);
	}
}

/* sets a red emit color */
void set_red_list()
{
	GLfloat emit[] = { 1.0, 0.0, 0.0, 1.0 };
	GLfloat specular[] = { 1.0, 0.0, 0.0, 0.5 };
	GLfloat mat_diffuse[] = { 1.0, 0.5, 0.5, 0.5 };
	
	dl->red_list = glGenLists(1);
	if (dl->red_list != 0)
	{
		glNewList(dl->red_list, GL_COMPILE);
		glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, mat_diffuse);
		glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, emit);
		glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specular);
		glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, 50.0);
		glEndList();
	}
	else
	{
		fprintf(stderr, "Unable to initialize red display list, aborting\n");
		exit(1);
	}
}

/* sets a blue emit color */
void set_blue_list()
{
	GLfloat emit[] = { 0.0, 0.0, 0.2, 1.0 };
	GLfloat specular[] = { 0.0, 0.0, 0.2, 1.0 };
	GLfloat mat_diffuse[] = { 0.0, 0.0, 0.2, 1.0 };
	
	dl->blue_list = glGenLists(1);
	if (dl->blue_list != 0)
	{
		glNewList(dl->blue_list, GL_COMPILE);
		glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, mat_diffuse);
		glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, emit);
		glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specular);
		glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, 50.0);
		glEndList();
	}
	else
	{
		fprintf(stderr, "Unable to initialize blue display list, aborting\n");
		exit(1);
	}
}

/* sets explosion color */
void set_explosion_list()
{
	GLfloat emit[] = { 1.0, 0.0, 0.0, 1.0 };
	GLfloat specular[] = { 1.0, 0.1, 0.0, 0.6 };
	GLfloat mat_diffuse[] = { 1.0, 0.1, 0.0, 0.6 };
	
	dl->explosion_list = glGenLists(1);
	if (dl->explosion_list != 0)
	{
		glNewList(dl->explosion_list, GL_COMPILE);
		glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, mat_diffuse);
		glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, emit);
		glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specular);
		glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, 50.0);
		glEndList();
	}
	else
	{
		fprintf(stderr, "Unable to initialize explosion display list, aborting\n");
		exit(1);
	}
}

/* Sets the color for the shots */
void set_shots_material()
{
//	GLfloat emit[] = { 0.0, 0.0, 0.3, 1.0 };
//	GLfloat specular[] = { 0.1, 0.0, 0.5, 1.0 };
//	GLfloat mat_diffuse[] = { 0.1, 0.0, 0.5, 1.0 };
	GLfloat emit[] = { 0.95, 0.95, 0.95, 1.0 };
	GLfloat specular[] = { 1.0, 1.0, 1.0, 1.0 };
	GLfloat mat_diffuse[] = { 1.0, 1.0, 1.0, 1.0 };
	
	dl->shots_list = glGenLists(1);
	if (dl->shots_list != 0)
	{
		glNewList(dl->shots_list, GL_COMPILE);
		glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, mat_diffuse);
		glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, emit);
		glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specular);
		glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, 80.0);
		glEndList();
	}
	else
	{
		fprintf(stderr, "Unable to initialize shots material display list, aborting\n");
		exit(1);
	}
}

/* Sets the display list to show the crosshair on screen */
void draw_crosshair()
{
	GLfloat centerx = ((GLfloat)state->windowx)/2.0;
	GLfloat centery = ((GLfloat)state->windowy)/2.0;
	GLfloat coord = 4.0;
	GLfloat offset = 10.0;

	glPushMatrix();

	glLoadIdentity();

	glCallList(dl->crosshair_list);
	
	glBegin(GL_LINES);
	/* top */
	glVertex2f(centerx, centery - coord - offset);
	glVertex2f(centerx, centery - offset);
	glVertex2f(centerx - 4.0, centery - offset);
	glVertex2f(centerx + 4.0, centery - offset);
	glVertex2f(centerx - 4.0, centery - offset - 4.0);
	glVertex2f(centerx + 4.0, centery - offset - 4.0);

	/* bottom */
	glVertex2f(centerx, centery + coord + offset);
	glVertex2f(centerx, centery + offset);
	glVertex2f(centerx - 4.0, centery + offset);
	glVertex2f(centerx + 4.0, centery + offset);
	glVertex2f(centerx - 4.0, centery + offset + 4.0);
	glVertex2f(centerx + 4.0, centery + offset + 4.0);

	/* left */
	glVertex2f(centerx - coord - offset, centery);
	glVertex2f(centerx - offset, centery);
	glVertex2f(centerx - offset - 4.0, centery + 4.0);
	glVertex2f(centerx - offset - 4.0, centery - 4.0);
	glVertex2f(centerx - offset, centery + 4.0);
	glVertex2f(centerx - offset, centery - 4.0);

	/* right */
	glVertex2f(centerx + coord + offset, centery);
	glVertex2f(centerx + offset, centery);
	glVertex2f(centerx + offset + 4.0, centery + 4.0);
	glVertex2f(centerx + offset + 4.0, centery - 4.0);
	glVertex2f(centerx + offset, centery + 4.0);
	glVertex2f(centerx + offset, centery - 4.0);

	glEnd();

	glCallList(dl->emit_off_list);
	
	glPopMatrix();
}

/* draw a textured crate with center at the origin */
void set_crate()
{
	dl->crate_list = glGenLists(1);
	if (dl->crate_list != 0)
	{
		glNewList(dl->crate_list, GL_COMPILE);
		//glCallList(dl->white_list);
		glPushMatrix();
		glEnable(GL_TEXTURE_2D);
		
		glBindTexture(GL_TEXTURE_2D, tex->tex_names[CRATE_WARN]);
		glBegin(GL_QUADS);
		// front
		glNormal3f(0.0, 0.0, -1.0);
		glTexCoord2f(0.0, 0.0); glVertex3f(1.0, -1.0, -1.0);
		glTexCoord2f(0.0, 1.0); glVertex3f(1.0, 1.0, -1.0);
		glTexCoord2f(1.0, 1.0); glVertex3f(-1.0, 1.0, -1.0);
		glTexCoord2f(1.0, 0.0); glVertex3f(-1.0, -1.0, -1.0);
		glEnd();
		// right
		glBindTexture(GL_TEXTURE_2D, tex->tex_names[CRATE_VENTS]);
		glBegin(GL_QUADS);
		glNormal3f(-1.0, 0.0, 0.0);
		glTexCoord2f(0.0, 0.0); glVertex3f(-1.0, -1.0, -1.0);
		glTexCoord2f(0.0, 1.0); glVertex3f(-1.0, +1.0, -1.0);
		glTexCoord2f(1.0, 1.0); glVertex3f(-1.0, +1.0, +1.0);
		glTexCoord2f(1.0, 0.0); glVertex3f(-1.0, -1.0, +1.0);
		glEnd();
		// back
		glBindTexture(GL_TEXTURE_2D, tex->tex_names[CRATE_WARN]);
		glBegin(GL_QUADS);
		glNormal3f(0.0, 0.0, 1.0);
		glTexCoord2f(0.0, 0.0); glVertex3f(-1.0, -1.0, +1.0);
		glTexCoord2f(0.0, 1.0); glVertex3f(-1.0, +1.0, +1.0);
		glTexCoord2f(1.0, 1.0); glVertex3f(1.0, +1.0, +1.0);
		glTexCoord2f(1.0, 0.0); glVertex3f(1.0, -1.0, +1.0);
		glEnd();
		// left
		glBindTexture(GL_TEXTURE_2D, tex->tex_names[CRATE_RCKTS]);
		glBegin(GL_QUADS);
		glNormal3f(1.0, 0.0, 0.0);
		glTexCoord2f(0.0, 0.0); glVertex3f(1.0, -1.0, +1.0);
		glTexCoord2f(0.0, 1.0); glVertex3f(1.0, +1.0, +1.0);
		glTexCoord2f(1.0, 1.0); glVertex3f(1.0, +1.0, -1.0);
		glTexCoord2f(1.0, 0.0); glVertex3f(1.0, -1.0, -1.0);
		glEnd();
		// top
		glBindTexture(GL_TEXTURE_2D, tex->tex_names[CRATE_EXPLO]);
		glBegin(GL_QUADS);
		glNormal3f(0.0, 1.0, 0.0);
		glTexCoord2f(0.0, 0.0); glVertex3f(1.0, +1.0, -1.0);
		glTexCoord2f(0.0, 1.0); glVertex3f(1.0, +1.0, +1.0);
		glTexCoord2f(1.0, 1.0); glVertex3f(-1.0, +1.0, +1.0);
		glTexCoord2f(1.0, 0.0); glVertex3f(-1.0, +1.0, -1.0);
		glEnd();
		// bottom
		glBindTexture(GL_TEXTURE_2D, tex->tex_names[CRATE_EXPLO]);
		glBegin(GL_QUADS);
		glNormal3f(0.0, -1.0, 0.0);
		glTexCoord2f(0.0, 0.0); glVertex3f(-1.0, -1.0, +1.0);
		glTexCoord2f(0.0, 1.0); glVertex3f(1.0, -1.0, +1.0);
		glTexCoord2f(1.0, 1.0); glVertex3f(1.0, -1.0, -1.0);
		glTexCoord2f(1.0, 0.0); glVertex3f(-1.0, -1.0, -1.0);
		glEnd();
	
		glDisable(GL_TEXTURE_2D);
		glPopMatrix();
		glEndList();
	}
	else
	{
		fprintf(stderr, "Unable to initialize crate display list, aborting\n");
		exit(1);
	}
}

/* Sets compass display list */
void set_compass_list()
{
	dl->compass_list = glGenLists(1);
	if (dl->compass_list != 0)
	{
		glNewList(dl->compass_list, GL_COMPILE);
		
		//glCallList(dl->white_list);
		glEnable(GL_TEXTURE_2D);
		glBindTexture(GL_TEXTURE_2D, tex->tex_names[COMPASS]);
		glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
		gluQuadricTexture(qobj, GL_TRUE);
		gluQuadricOrientation(qobj, GLU_INSIDE);
		gluQuadricDrawStyle(qobj, GLU_FILL);
		gluQuadricNormals(qobj, GLU_SMOOTH);
		
		gluCylinder(qobj, 0.1, 0.1, 0.04, 15, 5);
		
		glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
		glDisable(GL_TEXTURE_2D);
		glEndList();
	}
	else
	{
		fprintf(stderr, "Unable to initialize compass display list, aborting\n");
		exit(1);
	}
}

/* Sets barrel display list */
void set_barrel_list()
{
	GLdouble radius = 0.8;
	GLdouble height = 2.3;
		
	dl->barrel_list = glGenLists(1);
	if (dl->barrel_list != 0)
	{
		glNewList(dl->barrel_list, GL_COMPILE);
		
		//glCallList(dl->white_list);
		glEnable(GL_TEXTURE_2D);
		
		glBindTexture(GL_TEXTURE_2D, tex->tex_names[BARREL_SIDE]);
		gluQuadricTexture(qobj, GL_TRUE);
		gluQuadricOrientation(qobj, GLU_INSIDE);
		gluQuadricDrawStyle(qobj, GLU_FILL);
		gluQuadricNormals(qobj, GLU_SMOOTH);
		gluCylinder(qobj, radius, radius, height, 20, 20);
		glBindTexture(GL_TEXTURE_2D, tex->tex_names[BARREL_TOP]);
		gluQuadricOrientation(qobj, GLU_OUTSIDE);
		gluDisk(qobj, 0.0, radius, 20, 20);
		glTranslated(0.0, 0.0, height);
		glBindTexture(GL_TEXTURE_2D, tex->tex_names[BARREL_TOP]);
		gluQuadricOrientation(qobj, GLU_INSIDE);
		gluDisk(qobj, 0.0, radius, 20, 20);

		glDisable(GL_TEXTURE_2D);
		glEndList();
	}
	else
	{
		fprintf(stderr, "Unable to initialize barrel display list, aborting\n");
		exit(1);
	}
}

/* Sets (sub) fence display list */
/* Draws one element (square) of the fence */
void set_subfence_list()
{
	dl->sub_fence_list = glGenLists(1);
	if (dl->sub_fence_list != 0)
	{
		glNewList(dl->sub_fence_list, GL_COMPILE);
		
		//glCallList(dl->white_list);
		glEnable(GL_TEXTURE_2D);
		glDisable(GL_CULL_FACE);
		
		glPushMatrix();
		glBindTexture(GL_TEXTURE_2D, tex->tex_names[FENCE]);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		
		glBegin(GL_QUADS);
		glNormal3f(0.0, 0.0, -1.0);
		glTexCoord2f(0.0, 0.0); glVertex3f(0.0, 0.0, 0.0);
		glTexCoord2f(0.0, 1.0); glVertex3f(0.0, 2.0, 0.0);
		glTexCoord2f(1.0, 1.0); glVertex3f(-2.0, 2.0, 0.0);
		glTexCoord2f(1.0, 0.0); glVertex3f(-2.0, 0.0, 0.0);
		glEnd();
		
		glPopMatrix();
		
		glEnable(GL_CULL_FACE);
		glDisable(GL_TEXTURE_2D);
		glEndList();
	}
	else
	{
		fprintf(stderr, "Unable to initialize sub fence display list, aborting\n");
		exit(1);
	}
}

/* Sets fence display list */
/* Draws the whole fence using the sublist */
void set_fence_list()
{
	GLdouble i, j;
	
	dl->fence_list = glGenLists(1);
	if (dl->fence_list != 0)
	{
		glNewList(dl->fence_list, GL_COMPILE);
		
		glDepthMask(GL_FALSE);
		//glCallList(dl->white_list);
		
		for(i = 31.0; i >= 11.0; i-=2.0)
		{
			for (j = 0.1; j <= 2.1; j+=2.0)
			{	
				glPushMatrix();
				glRotated(90.0, 0.0, 1.0, 0.0);
				glTranslated(i, j, -10.0);
				glCallList(dl->sub_fence_list);
				glPopMatrix();
			}
		}
		for(i = -10.0; i >= -18.0; i-=2.0)
		{
			for (j = 0.1; j <= 2.1; j+=2.0)
			{	
				glPushMatrix();
				glTranslated(i, j, -9.0);
				glCallList(dl->sub_fence_list);
				glPopMatrix();
			}
		}
		
		glDepthMask(GL_TRUE);
		glEndList();
	}
	else
	{
		fprintf(stderr, "Unable to initialize fence display list, aborting\n");
		exit(1);
	}
}

/* Sets post display list */
/* Draws one post */
void set_post_list()
{
	GLdouble radius = 0.3;
	GLdouble height = 2.0;
	
	dl->post_list = glGenLists(1);
	if (dl->post_list != 0)
	{
		glNewList(dl->post_list, GL_COMPILE);
		
		//glCallList(dl->white_list);
		glEnable(GL_TEXTURE_2D);
		glBindTexture(GL_TEXTURE_2D, tex->tex_names[BEAM_METAL]);
		
		glBegin(GL_QUADS);
		// front
		glNormal3f(0.0, 0.0, -1.0);
		glTexCoord2f(0.0, 0.0); glVertex3f(0.0, 0.0, 0.0);
		glTexCoord2f(0.0, 1.0); glVertex3f(0.0, height, 0.0);
		glTexCoord2f(1.0, 1.0); glVertex3f(-radius, height, 0.0);
		glTexCoord2f(1.0, 0.0); glVertex3f(-radius, 0.0, 0.0);
		// right
		glNormal3f(-1.0, 0.0, 0.0);
		glTexCoord2f(0.0, 0.0); glVertex3f(-radius, 0.0, 0.0);
		glTexCoord2f(0.0, 1.0); glVertex3f(-radius, height, 0.0);
		glTexCoord2f(1.0, 1.0); glVertex3f(-radius, height, radius);
		glTexCoord2f(1.0, 0.0); glVertex3f(-radius, 0.0, radius);
		// back
		glNormal3f(0.0, 0.0, 1.0);
		glTexCoord2f(0.0, 0.0); glVertex3f(-radius, 0.0, radius);
		glTexCoord2f(0.0, 1.0); glVertex3f(-radius, height, radius);
		glTexCoord2f(1.0, 1.0); glVertex3f(0.0, height, radius);
		glTexCoord2f(1.0, 0.0); glVertex3f(0.0, 0.0, radius);
		// left
		glNormal3f(1.0, 0.0, 0.0);
		glTexCoord2f(0.0, 0.0); glVertex3f(0.0, 0.0, radius);
		glTexCoord2f(0.0, 1.0); glVertex3f(0.0, height, radius);
		glTexCoord2f(1.0, 1.0); glVertex3f(0.0, height, 0.0);
		glTexCoord2f(1.0, 0.0); glVertex3f(0.0, 0.0, 0.0);
		// top
		glNormal3f(0.0, 1.0, 0.0);
		glTexCoord2f(0.0, 0.0); glVertex3f(0.0, height, 0.0);
		glTexCoord2f(0.0, 1.0); glVertex3f(0.0, height, radius);
		glTexCoord2f(1.0, 1.0); glVertex3f(-radius, height, radius);
		glTexCoord2f(1.0, 0.0); glVertex3f(-radius, height, 0.0);
		// bottom
		glNormal3f(0.0, -1.0, 0.0);
		glTexCoord2f(0.0, 0.0); glVertex3f(0.0, 0.0, radius);
		glTexCoord2f(0.0, 1.0); glVertex3f(0.0, 0.0, 0.0);
		glTexCoord2f(1.0, 1.0); glVertex3f(-radius, 0.0, 0.0);
		glTexCoord2f(1.0, 0.0); glVertex3f(-radius, 0.0, radius);
		glEnd();
		
		glDisable(GL_TEXTURE_2D);
		glEndList();
	}
	else
	{
		fprintf(stderr, "Unable to initialize post display list, aborting\n");
		exit(1);
	}
}

/* Sets post display list */
/* Draws one post */
void set_double_post_list()
{
	dl->double_post_list = glGenLists(1);
	if (dl->double_post_list != 0)
	{
		glNewList(dl->double_post_list, GL_COMPILE);
		glCallList(dl->post_list);
		glTranslatef(0.0, 2.0, 0.0);
		glCallList(dl->post_list);
		glEndList();
	}
	else
	{
		fprintf(stderr, "Unable to initialize double post display list, aborting\n");
		exit(1);
	}
}

/* Draws control console */
void set_console_list()
{
	dl->console_list = glGenLists(1);
	if (dl->console_list != 0)
	{
		glNewList(dl->console_list, GL_COMPILE);
		
		glPushMatrix();
		//glCallList(dl->white_list);
		
		glEnable(GL_TEXTURE_2D);
		glBindTexture(GL_TEXTURE_2D, tex->tex_names[METAL]);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);

		// base

		glBegin(GL_QUADS);

		// front
		glNormal3f(0.0, 0.0, -1.0);
		glTexCoord2f(0.0, 0.0); glVertex3f(5.5, 0.0, 10.0);
		glTexCoord2f(0.0, 1.0); glVertex3f(5.5, 1.79, 10.0);
		glTexCoord2f(1.0, 1.0); glVertex3f(4.5, 1.79, 10.0);
		glTexCoord2f(1.0, 0.0); glVertex3f(4.5, 0.0, 10.0);

		// right
		glNormal3f(-1.0, 0.0, 0.0);
		glTexCoord2f(0.0, 0.0); glVertex3f(4.5, 0.0, 10.0);
		glTexCoord2f(0.0, 1.0); glVertex3f(4.5, 1.8, 10.0);
		glTexCoord2f(1.0, 1.0); glVertex3f(4.5, 1.8, 10.5);
		glTexCoord2f(1.0, 0.0); glVertex3f(4.5, 0.0, 10.5);

		// back
		glNormal3f(0.0, 0.0, 1.0);
		glTexCoord2f(0.0, 0.0); glVertex3f(4.5, 0.0, 10.5);
		glTexCoord2f(0.0, 1.0); glVertex3f(4.5, 1.83, 10.5);
		glTexCoord2f(1.0, 1.0); glVertex3f(5.5, 1.83, 10.5);
		glTexCoord2f(1.0, 0.0); glVertex3f(5.5, 0.0, 10.5);

		// left
		glNormal3f(1.0, 0.0, 0.0);
		glTexCoord2f(0.0, 0.0); glVertex3f(5.5, 0.0, 10.5);
		glTexCoord2f(0.0, 1.0); glVertex3f(5.5, 1.8, 10.5);
		glTexCoord2f(1.0, 1.0); glVertex3f(5.5, 1.8, 10.0);
		glTexCoord2f(1.0, 0.0); glVertex3f(5.5, 0.0, 10.0);

		glEnd();

		// console

		glTranslated(5.0, 1.8, 10.0);
		glRotated(140.0, 1.0, 0.0, 0.0);

		glPushMatrix();

		glBindTexture(GL_TEXTURE_2D, tex->tex_names[METAL]);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		
		glBegin(GL_QUADS);
		
		glNormal3f(0.0, 1.0, 0.0);

		// front
		glTexCoord2f(0.0, 0.0); glVertex3f(1.2, 0.0, -0.8);
		glTexCoord2f(0.0, 1.0); glVertex3f(1.0, 0.3, -0.6);
		glTexCoord2f(1.0, 1.0); glVertex3f(-1.0, 0.3, -0.6);
		glTexCoord2f(1.0, 0.0); glVertex3f(-1.2, 0.0, -0.8);

		// right
		glTexCoord2f(0.0, 0.0); glVertex3f(-1.2, 0.0, -0.8);
		glTexCoord2f(0.0, 1.0); glVertex3f(-1.0, 0.3, -0.6);
		glTexCoord2f(1.0, 1.0); glVertex3f(-1.0, 0.3, 0.6);
		glTexCoord2f(1.0, 0.0); glVertex3f(-1.2, 0.0, 0.8);

		// back
		glTexCoord2f(0.0, 0.0); glVertex3f(-1.2, 0.0, 0.8);
		glTexCoord2f(0.0, 1.0); glVertex3f(-1.0, 0.3, 0.6);
		glTexCoord2f(1.0, 1.0); glVertex3f(1.0, 0.3, 0.6);
		glTexCoord2f(1.0, 0.0); glVertex3f(1.2, 0.0, 0.8);

		// left
		glTexCoord2f(0.0, 0.0); glVertex3f(1.2, 0.0, 0.8);
		glTexCoord2f(0.0, 1.0); glVertex3f(1.0, 0.3, 0.6);
		glTexCoord2f(1.0, 1.0); glVertex3f(1.0, 0.3, -0.6);
		glTexCoord2f(1.0, 0.0); glVertex3f(1.2, 0.0, -0.8);

		// top
		glTexCoord2f(0.0, 0.0); glVertex3f(1.0, 0.3, -0.6);
		glTexCoord2f(0.0, 1.0); glVertex3f(1.0, 0.3, 0.6);
		glTexCoord2f(1.0, 1.0); glVertex3f(-1.0, 0.3, 0.6);
		glTexCoord2f(1.0, 0.0); glVertex3f(-1.0, 0.3, -0.6);

		glEnd();
		
		glPopMatrix();
		glDisable(GL_TEXTURE_2D);
		glPopMatrix();
		glEndList();
	}
	else
	{
		fprintf(stderr, "Unable to initialize console display list, aborting\n");
		exit(1);
	}
}

/* Draws a textured container with center at the origin */
void set_container_list()
{
	dl->container_list = glGenLists(1);
	if (dl->container_list != 0)
	{
		glNewList(dl->container_list, GL_COMPILE);

		//glCallList(dl->white_list);
		
		glEnable(GL_TEXTURE_2D);
		glBindTexture(GL_TEXTURE_2D, tex->tex_names[CONTAINER]);
		
		glBegin(GL_QUADS);
		// front
		glNormal3f(0.0, 0.0, -1.0);
		glTexCoord2f(0.0, 0.0); glVertex3f(1.0, -1.0, -1.0);
		glTexCoord2f(0.0, 1.0); glVertex3f(1.0, 1.0, -1.0);
		glTexCoord2f(1.0, 1.0); glVertex3f(-1.0, 1.0, -1.0);
		glTexCoord2f(1.0, 0.0); glVertex3f(-1.0, -1.0, -1.0);
		
		// right
		glNormal3f(-1.0, 0.0, 0.0);
		glTexCoord2f(0.0, 0.0); glVertex3f(-1.0, -1.0, -1.0);
		glTexCoord2f(0.0, 1.0); glVertex3f(-1.0, +1.0, -1.0);
		glTexCoord2f(1.0, 1.0); glVertex3f(-1.0, +1.0, +1.0);
		glTexCoord2f(1.0, 0.0); glVertex3f(-1.0, -1.0, +1.0);
	
		// back
		glNormal3f(0.0, 0.0, 1.0);
		glTexCoord2f(0.0, 0.0); glVertex3f(-1.0, -1.0, +1.0);
		glTexCoord2f(0.0, 1.0); glVertex3f(-1.0, +1.0, +1.0);
		glTexCoord2f(1.0, 1.0); glVertex3f(1.0, +1.0, +1.0);
		glTexCoord2f(1.0, 0.0); glVertex3f(1.0, -1.0, +1.0);

		// left
		glNormal3f(1.0, 0.0, 0.0);
		glTexCoord2f(0.0, 0.0); glVertex3f(1.0, -1.0, +1.0);
		glTexCoord2f(0.0, 1.0); glVertex3f(1.0, +1.0, +1.0);
		glTexCoord2f(1.0, 1.0); glVertex3f(1.0, +1.0, -1.0);
		glTexCoord2f(1.0, 0.0); glVertex3f(1.0, -1.0, -1.0);

		// top
		glNormal3f(0.0, 1.0, 0.0);
		glTexCoord2f(0.0, 0.0); glVertex3f(1.0, +1.0, -1.0);
		glTexCoord2f(0.0, 1.0); glVertex3f(1.0, +1.0, +1.0);
		glTexCoord2f(1.0, 1.0); glVertex3f(-1.0, +1.0, +1.0);
		glTexCoord2f(1.0, 0.0); glVertex3f(-1.0, +1.0, -1.0);
		
		// bottom
		glNormal3f(0.0, -1.0, 0.0);
		glTexCoord2f(0.0, 0.0); glVertex3f(-1.0, -1.0, +1.0);
		glTexCoord2f(0.0, 1.0); glVertex3f(1.0, -1.0, +1.0);
		glTexCoord2f(1.0, 1.0); glVertex3f(1.0, -1.0, -1.0);
		glTexCoord2f(1.0, 0.0); glVertex3f(-1.0, -1.0, -1.0);
		glEnd();
	
		glDisable(GL_TEXTURE_2D);
		glEndList();
	}
	else
	{
		fprintf(stderr, "Unable to initialize container display list, aborting\n");
		exit(1);
	}
}

/* Sets up door display list */
void set_door_list()
{
	dl->door_list = glGenLists(1);
	if (dl->door_list != 0)
	{
		glNewList(dl->door_list, GL_COMPILE);

		//glCallList(dl->white_list);
		
		glEnable(GL_TEXTURE_2D);
		glBindTexture(GL_TEXTURE_2D, tex->tex_names[DOOR_METAL]);
		
		glPushMatrix();

		glScaled(1.3, 1.6, 1.3);

		glBegin(GL_QUADS);

		// bottom left
		glNormal3f(0.0, 0.0, -1.0);
		glTexCoord2f(0.25, 0.0); glVertex3f(1.0, 0.0, 0.0);
		glTexCoord2f(0.0, 0.5); glVertex3f(2.0, 1.5, 0.0);
		glTexCoord2f(0.5, 0.5); glVertex3f(0.0, 1.5, 0.0);
		glTexCoord2f(0.5, 0.0); glVertex3f(0.0, 0.0, 0.0);

		// bottom right
		glTexCoord2f(0.5, 0.0); glVertex3f(0.0, 0.0, 0.0);
		glTexCoord2f(0.5, 0.5); glVertex3f(0.0, 1.5, 0.0);
		glTexCoord2f(1.0, 0.5); glVertex3f(-2.0, 1.5, 0.0);
		glTexCoord2f(0.75, 0.0); glVertex3f(-1.0, 0.0, 0.0);

		// upper left
		glTexCoord2f(0.0, 0.5); glVertex3f(2.0, 1.5, 0.0);
		glTexCoord2f(0.25, 1.0); glVertex3f(1.0, 3.0, 0.0);
		glTexCoord2f(0.5, 1.0); glVertex3f(0.0, 3.0, 0.0);
		glTexCoord2f(0.5, 0.5); glVertex3f(0.0, 1.5, 0.0);

		// upper right
		glTexCoord2f(0.5, 0.5); glVertex3f(0.0, 1.5, 0.0);
		glTexCoord2f(0.5, 1.0); glVertex3f(0.0, 3.0, 0.0);
		glTexCoord2f(0.75, 1.0); glVertex3f(-1.0, 3.0, 0.0);
		glTexCoord2f(1.0, 0.5); glVertex3f(-2.0, 1.5, 0.0);
		glEnd();

		glBindTexture(GL_TEXTURE_2D, tex->tex_names[METAL]);
		
		glBegin(GL_QUADS);

		// bottom left
		glNormal3f(1.0, 0.0, 0.0);
		glTexCoord2f(0.0, 0.0); glVertex3f(1.0, 0.0, 0.2);
		glTexCoord2f(0.0, 1.0); glVertex3f(2.0, 1.5, 0.2);
		glTexCoord2f(1.0, 1.0); glVertex3f(2.0, 1.5, 0.0);
		glTexCoord2f(1.0, 0.0); glVertex3f(1.0, 0.0, 0.0);
		
		// upper left
		glTexCoord2f(0.0, 0.0); glVertex3f(2.0, 1.5, 0.2);
		glTexCoord2f(0.0, 1.0); glVertex3f(1.0, 3.0, 0.2);
		glTexCoord2f(1.0, 1.0); glVertex3f(1.0, 3.0, 0.0);
		glTexCoord2f(1.0, 0.0); glVertex3f(2.0, 1.5, 0.0);
		
		// top
		glNormal3f(0.0, 1.0, 0.0);
		glTexCoord2f(0.0, 0.0); glVertex3f(1.0, 3.0, 0.2);
		glTexCoord2f(0.0, 1.0); glVertex3f(-1.0, 3.0, 0.2);
		glTexCoord2f(1.0, 1.0); glVertex3f(-1.0, 3.0, 0.0);
		glTexCoord2f(1.0, 0.0); glVertex3f(1.0, 3.0, 0.0);
		
		// upper right
		glNormal3f(-1.0, 0.0, 0.0);
		glTexCoord2f(0.0, 0.0); glVertex3f(-2.0, 1.5, 0.0);
		glTexCoord2f(0.0, 1.0); glVertex3f(-1.0, 3.0, 0.0);
		glTexCoord2f(1.0, 1.0); glVertex3f(-1.0, 3.0, 0.2);
		glTexCoord2f(1.0, 0.0); glVertex3f(-2.0, 1.5, 0.2);
		
		// bottom right
		glTexCoord2f(0.0, 0.0); glVertex3f(-1.0, 0.0, 0.0);
		glTexCoord2f(0.0, 1.0); glVertex3f(-2.0, 1.5, 0.0);
		glTexCoord2f(1.0, 1.0); glVertex3f(-2.0, 1.5, 0.2);
		glTexCoord2f(1.0, 0.0); glVertex3f(-1.0, 0.0, 0.2);
		glEnd();

		glPopMatrix();
		
		glDisable(GL_TEXTURE_2D);
		glEndList();
	}
	else
	{
		fprintf(stderr, "Unable to initialize door display list, aborting\n");
		exit(1);
	}
}

/* draw a textured fan object with center at the origin */
void set_fan_list()
{
	dl->fan_list = glGenLists(1);
	if (dl->fan_list != 0)
	{
		glNewList(dl->fan_list, GL_COMPILE);
		//glCallList(dl->white_list);
		glEnable(GL_TEXTURE_2D);
		glBindTexture(GL_TEXTURE_2D, tex->tex_names[FAN]);
		
		// front
		glBegin(GL_QUADS);
		glNormal3f(0.0, 0.0, -1.0);
		glTexCoord2f(0.0, 0.0); glVertex3f(1.0, -1.0, -1.0);
		glTexCoord2f(0.0, 1.0); glVertex3f(1.0, 1.0, -1.0);
		glTexCoord2f(1.0, 1.0); glVertex3f(-1.0, 1.0, -1.0);
		glTexCoord2f(1.0, 0.0); glVertex3f(-1.0, -1.0, -1.0);
		glEnd();
		
		glBindTexture(GL_TEXTURE_2D, tex->tex_names[VENT]);
		// back
		glBegin(GL_QUADS);
		glNormal3f(0.0, 0.0, 1.0);
		glTexCoord2f(0.0, 0.0); glVertex3f(-1.0, -1.0, +1.0);
		glTexCoord2f(0.0, 1.0); glVertex3f(-1.0, +1.0, +1.0);
		glTexCoord2f(1.0, 1.0); glVertex3f(1.0, +1.0, +1.0);
		glTexCoord2f(1.0, 0.0); glVertex3f(1.0, -1.0, +1.0);
		glEnd();
		
		glBindTexture(GL_TEXTURE_2D, tex->tex_names[DARK_METAL]);
		// right
		glBegin(GL_QUADS);
		glNormal3f(-1.0, 0.0, 0.0);
		glTexCoord2f(0.0, 0.0); glVertex3f(-1.0, -1.0, -1.0);
		glTexCoord2f(0.0, 1.0); glVertex3f(-1.0, +1.0, -1.0);
		glTexCoord2f(1.0, 1.0); glVertex3f(-1.0, +1.0, +1.0);
		glTexCoord2f(1.0, 0.0); glVertex3f(-1.0, -1.0, +1.0);
		glEnd();
		
		// left
		glBegin(GL_QUADS);
		glNormal3f(1.0, 0.0, 0.0);
		glTexCoord2f(0.0, 0.0); glVertex3f(1.0, -1.0, +1.0);
		glTexCoord2f(0.0, 1.0); glVertex3f(1.0, +1.0, +1.0);
		glTexCoord2f(1.0, 1.0); glVertex3f(1.0, +1.0, -1.0);
		glTexCoord2f(1.0, 0.0); glVertex3f(1.0, -1.0, -1.0);
		glEnd();

		// top
		glBegin(GL_QUADS);
		glNormal3f(0.0, 1.0, 0.0);
		glTexCoord2f(0.0, 0.0); glVertex3f(1.0, +1.0, -1.0);
		glTexCoord2f(0.0, 1.0); glVertex3f(1.0, +1.0, +1.0);
		glTexCoord2f(1.0, 1.0); glVertex3f(-1.0, +1.0, +1.0);
		glTexCoord2f(1.0, 0.0); glVertex3f(-1.0, +1.0, -1.0);
		glEnd();
		
		// bottom
		glBegin(GL_QUADS);
		glNormal3f(0.0, -1.0, 0.0);
		glTexCoord2f(0.0, 0.0); glVertex3f(-1.0, -1.0, +1.0);
		glTexCoord2f(0.0, 1.0); glVertex3f(1.0, -1.0, +1.0);
		glTexCoord2f(1.0, 1.0); glVertex3f(1.0, -1.0, -1.0);
		glTexCoord2f(1.0, 0.0); glVertex3f(-1.0, -1.0, -1.0);
		glEnd();
	
		glDisable(GL_TEXTURE_2D);
		glEndList();
	}
	else
	{
		fprintf(stderr, "Unable to initialize fan display list, aborting\n");
		exit(1);
	}
}

/* draw a textured panel object with center at the origin */
void set_panel_list()
{
	dl->panel_list = glGenLists(1);
	if (dl->panel_list != 0)
	{
		glNewList(dl->panel_list, GL_COMPILE);
		//glCallList(dl->white_list);
		glEnable(GL_TEXTURE_2D);
		glBindTexture(GL_TEXTURE_2D, tex->tex_names[PANEL]);
		
		glBegin(GL_QUADS);
		glNormal3f(0.0, 0.0, -1.0);
		glTexCoord2f(0.0, 0.0); glVertex3f(0.7, -0.93, 0.0);
		glTexCoord2f(0.0, 1.0); glVertex3f(0.7, 0.93, 0.0);
		glTexCoord2f(1.0, 1.0); glVertex3f(-0.7, 0.93, 0.0);
		glTexCoord2f(1.0, 0.0); glVertex3f(-0.7, -0.93, 0.0);
		glEnd();
	
		glBindTexture(GL_TEXTURE_2D, tex->tex_names[METAL]);
		
		glBegin(GL_QUADS);
		// left
		glNormal3f(1.0, 0.0, 0.0);
		glTexCoord2f(0.0, 0.0); glVertex3f(0.7, -0.93, 0.1);
		glTexCoord2f(0.0, 1.0); glVertex3f(0.7, 0.93, 0.1);
		glTexCoord2f(1.0, 1.0); glVertex3f(0.7, 0.93, 0.0);
		glTexCoord2f(1.0, 0.0); glVertex3f(0.7, -0.93, 0.0);

		// bottom
		glNormal3f(0.0, -1.0, 0.0);
		glTexCoord2f(0.0, 0.0); glVertex3f(-0.7, -0.93, 0.1);
		glTexCoord2f(0.0, 1.0); glVertex3f(0.7, -0.93, 0.1);
		glTexCoord2f(1.0, 1.0); glVertex3f(0.7, -0.93, 0.0);
		glTexCoord2f(1.0, 0.0); glVertex3f(-0.7, -0.93, 0.0);

		// right
		glNormal3f(-1.0, 0.0, 0.0);
		glTexCoord2f(0.0, 0.0); glVertex3f(-0.7, -0.93, 0.0);
		glTexCoord2f(0.0, 1.0); glVertex3f(-0.7, 0.93, 0.0);
		glTexCoord2f(1.0, 1.0); glVertex3f(-0.7, 0.93, 0.1);
		glTexCoord2f(1.0, 0.0); glVertex3f(-0.7, -0.93, 0.1);

		// top
		glNormal3f(0.0, 1.0, 0.0);
		glTexCoord2f(0.0, 0.0); glVertex3f(-0.7, 0.93, 0.0);
		glTexCoord2f(0.0, 1.0); glVertex3f(0.7, 0.93, 0.0);
		glTexCoord2f(1.0, 1.0); glVertex3f(0.7, 0.93, 0.1);
		glTexCoord2f(1.0, 0.0); glVertex3f(-0.7, 0.93, 0.1);
		glEnd();
		
		glDisable(GL_TEXTURE_2D);
		glEndList();
	}
	else
	{
		fprintf(stderr, "Unable to initialize panel display list, aborting\n");
		exit(1);
	}
}

/* display list to draw ventilation tunnel */
void set_tunnel_list()
{
	GLfloat emit[] = { 0.1, 0.1, 0.1, 1.0 };
	GLfloat specular[] = { 0.1, 0.1, 0.1, 1.0 };
	GLfloat mat_diffuse[] = { 0.3, 0.3, 0.3, 1.0 };
	
	dl->tunnel_list = glGenLists(1);
	if (dl->tunnel_list != 0)
	{
		glNewList(dl->tunnel_list, GL_COMPILE);

		//glCallList(dl->white_list);
		glEnable(GL_TEXTURE_2D);
		glBindTexture(GL_TEXTURE_2D, tex->tex_names[METAL2]);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);

		glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, mat_diffuse);
		glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, emit);
		glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specular);
		glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, 50.0);
		
		// front
		glBegin(GL_TRIANGLES);
		glNormal3f(0.0, 0.0, -1.0);
		glTexCoord2f(0.0, 0.0); glVertex3f(1.0, -0.5, -0.5);
		glTexCoord2f(0.0, 1.0); glVertex3f(1.0, 0.5, -0.5);
		glTexCoord2f(0.5, 0.5); glVertex3f(0.0, 0.0, -0.6);
		
		glTexCoord2f(0.0, 1.0); glVertex3f(1.0, 0.5, -0.5);
		glTexCoord2f(1.0, 1.0); glVertex3f(-1.0, 0.5, -0.5);
		glTexCoord2f(0.5, 0.5); glVertex3f(0.0, 0.0, -0.6);

		glTexCoord2f(1.0, 1.0); glVertex3f(-1.0, 0.5, -0.5);
		glTexCoord2f(1.0, 0.0); glVertex3f(-1.0, -0.5, -0.5);
		glTexCoord2f(0.5, 0.5); glVertex3f(0.0, 0.0, -0.6);

		glTexCoord2f(1.0, 0.0); glVertex3f(-1.0, -0.5, -0.5);
		glTexCoord2f(0.0, 0.0); glVertex3f(1.0, -0.5, -0.5);
		glTexCoord2f(0.5, 0.5); glVertex3f(0.0, 0.0, -0.6);
		glEnd();
		
		// back
		glBegin(GL_TRIANGLES);
		glNormal3f(0.0, 0.0, 1.0);
		glTexCoord2f(0.0, 0.0); glVertex3f(-1.0, -0.5, 0.5);
		glTexCoord2f(0.0, 1.0); glVertex3f(-1.0, 0.5, 0.5);
		glTexCoord2f(0.5, 0.5); glVertex3f(0.0, 0.0, 0.6);
		
		glTexCoord2f(0.0, 1.0); glVertex3f(-1.0, 0.5, 0.5);
		glTexCoord2f(1.0, 1.0); glVertex3f(1.0, 0.5, 0.5);
		glTexCoord2f(0.5, 0.5); glVertex3f(0.0, 0.0, 0.6);

		glTexCoord2f(1.0, 1.0); glVertex3f(1.0, 0.5, 0.5);
		glTexCoord2f(1.0, 0.0); glVertex3f(1.0, -0.5, 0.5);
		glTexCoord2f(0.5, 0.5); glVertex3f(0.0, 0.0, 0.6);

		glTexCoord2f(1.0, 0.0); glVertex3f(1.0, -0.5, 0.5);
		glTexCoord2f(0.0, 0.0); glVertex3f(-1.0, -0.5, 0.5);
		glTexCoord2f(0.5, 0.5); glVertex3f(0.0, 0.0, 0.6);
		glEnd();
		
		// right
		glBegin(GL_QUADS);
		glNormal3f(-1.0, 0.0, 0.0);
		glTexCoord2f(0.0, 0.0); glVertex3f(-1.0, -0.5, -0.5);
		glTexCoord2f(0.0, 1.0); glVertex3f(-1.0, +0.5, -0.5);
		glTexCoord2f(1.0, 1.0); glVertex3f(-1.0, +0.5, +0.5);
		glTexCoord2f(1.0, 0.0); glVertex3f(-1.0, -0.5, +0.5);
		glEnd();
		
		// left
		glBegin(GL_QUADS);
		glNormal3f(1.0, 0.0, 0.0);
		glTexCoord2f(0.0, 0.0); glVertex3f(1.0, -0.5, +0.5);
		glTexCoord2f(0.0, 1.0); glVertex3f(1.0, +0.5, +0.5);
		glTexCoord2f(1.0, 1.0); glVertex3f(1.0, +0.5, -0.5);
		glTexCoord2f(1.0, 0.0); glVertex3f(1.0, -0.5, -0.5);
		glEnd();
		
		// top
		glBegin(GL_TRIANGLES);
		glNormal3f(0.0, 1.0, 0.0);
		glTexCoord2f(0.0, 0.0); glVertex3f(1.0, 0.5, -0.5);
		glTexCoord2f(0.0, 1.0); glVertex3f(1.0, 0.5, 0.5);
		glTexCoord2f(0.5, 0.5); glVertex3f(0.0, 0.6, 0.0);
		
		glTexCoord2f(0.0, 1.0); glVertex3f(1.0, 0.5, 0.5);
		glTexCoord2f(1.0, 1.0); glVertex3f(-1.0, 0.5, 0.5);
		glTexCoord2f(0.5, 0.5); glVertex3f(0.0, 0.6, 0.0);

		glTexCoord2f(1.0, 1.0); glVertex3f(-1.0, 0.5, 0.5);
		glTexCoord2f(1.0, 0.0); glVertex3f(-1.0, 0.5, -0.5);
		glTexCoord2f(0.5, 0.5); glVertex3f(0.0, 0.6, 0.0);

		glTexCoord2f(1.0, 0.0); glVertex3f(-1.0, 0.5, -0.5);
		glTexCoord2f(0.0, 0.0); glVertex3f(1.0, 0.5, -0.5);
		glTexCoord2f(0.5, 0.5); glVertex3f(0.0, 0.6, 0.0);
		glEnd();
		
		// bottom
		glBegin(GL_TRIANGLES);
		glNormal3f(0.0, -1.0, 0.0);
		glTexCoord2f(0.0, 0.0); glVertex3f(1.0, -0.5, 0.5);
		glTexCoord2f(0.0, 1.0); glVertex3f(1.0, -0.5, -0.5);
		glTexCoord2f(0.5, 0.5); glVertex3f(0.0, -0.6, 0.0);
		
		glTexCoord2f(0.0, 1.0); glVertex3f(1.0, -0.5, -0.5);
		glTexCoord2f(1.0, 1.0); glVertex3f(-1.0, -0.5, -0.5);
		glTexCoord2f(0.5, 0.5); glVertex3f(0.0, -0.6, 0.0);

		glTexCoord2f(1.0, 1.0); glVertex3f(-1.0, -0.5, -0.5);
		glTexCoord2f(1.0, 0.0); glVertex3f(-1.0, -0.5, 0.5);
		glTexCoord2f(0.5, 0.5); glVertex3f(0.0, -0.6, 0.0);

		glTexCoord2f(1.0, 0.0); glVertex3f(-1.0, -0.5, 0.5);
		glTexCoord2f(0.0, 0.0); glVertex3f(1.0, -0.5, 0.5);
		glTexCoord2f(0.5, 0.5); glVertex3f(0.0, -0.6, 0.0);
		glEnd();
	
		glDisable(GL_TEXTURE_2D);
		glCallList(dl->white_list);
		glEndList();
	}
	else
	{
		fprintf(stderr, "Unable to initialize tunnel display list, aborting\n");
		exit(1);
	}
}

/* display list to draw a red light object */
void set_redlight_list()
{
	dl->redlight_list = glGenLists(1);
	if (dl->redlight_list != 0)
	{
		glNewList(dl->redlight_list, GL_COMPILE);
		//glCallList(dl->white_list);
		glEnable(GL_TEXTURE_2D);
		glBindTexture(GL_TEXTURE_2D, tex->tex_names[METAL]);
		
		// left
		glBegin(GL_QUADS);
		glNormal3f(1.0, 0.0, 0.0);
		glTexCoord2f(0.0, 0.0); glVertex3f(0.25, -0.125, 0.25);
		glTexCoord2f(0.0, 1.0); glVertex3f(0.25, 0.125, 0.25);
		glTexCoord2f(1.0, 1.0); glVertex3f(0.25, 0.125, -0.25);
		glTexCoord2f(1.0, 0.0); glVertex3f(0.25, -0.125, -0.25);
		glEnd();
	
		// front
		glBegin(GL_QUADS);
		glNormal3f(0.0, 0.0, -1.0);
		glTexCoord2f(0.0, 0.0); glVertex3f(0.25, -0.125, -0.25);
		glTexCoord2f(0.0, 1.0); glVertex3f(0.25, 0.125, -0.25);
		glTexCoord2f(1.0, 1.0); glVertex3f(-0.25, 0.125, -0.25);
		glTexCoord2f(1.0, 0.0); glVertex3f(-0.25, -0.125, -0.25);
		glEnd();
	
		// right
		glBegin(GL_QUADS);
		glNormal3f(1.0, 0.0, 0.0);
		glTexCoord2f(0.0, 0.0); glVertex3f(-0.25, -0.125, -0.25);
		glTexCoord2f(0.0, 1.0); glVertex3f(-0.25, 0.125, -0.25);
		glTexCoord2f(1.0, 1.0); glVertex3f(-0.25, 0.125, 0.25);
		glTexCoord2f(1.0, 0.0); glVertex3f(-0.25, -0.125, 0.25);
		glEnd();
	
		// back
		glBegin(GL_QUADS);
		glNormal3f(0.0, 0.0, 1.0);
		glTexCoord2f(0.0, 0.0); glVertex3f(-0.25, -0.125, 0.25);
		glTexCoord2f(0.0, 1.0); glVertex3f(-0.25, 0.125, 0.25);
		glTexCoord2f(1.0, 1.0); glVertex3f(0.25, 0.125, 0.25);
		glTexCoord2f(1.0, 0.0); glVertex3f(0.25, -0.125, 0.25);
		glEnd();
	
		// bottom
		glBegin(GL_QUADS);
		glNormal3f(0.0, -1.0, 0.0);
		glTexCoord2f(0.0, 0.0); glVertex3f(0.25, -0.125, 0.25);
		glTexCoord2f(0.0, 1.0); glVertex3f(0.25, -0.125, -0.25);
		glTexCoord2f(1.0, 1.0); glVertex3f(-0.25, -0.125, -0.25);
		glTexCoord2f(1.0, 0.0); glVertex3f(-0.25, -0.125, 0.25);
		glEnd();
	
		glBindTexture(GL_TEXTURE_2D, tex->tex_names[REDLIGHT]);
		// top
		glBegin(GL_QUADS);
		glNormal3f(0.0, 1.0, 0.0);
		glTexCoord2f(0.0, 0.0); glVertex3f(0.25, 0.125, -0.25);
		glTexCoord2f(0.0, 1.0); glVertex3f(0.25, 0.125, 0.25);
		glTexCoord2f(1.0, 1.0); glVertex3f(-0.25, 0.125, 0.25);
		glTexCoord2f(1.0, 0.0); glVertex3f(-0.25, 0.125, -0.25);
		glEnd();
	
		glDisable(GL_TEXTURE_2D);
		glEndList();
	}
	else
	{
		fprintf(stderr, "Unable to initialize redlight display list, aborting\n");
		exit(1);
	}
}

/* Sets walls display list */
void set_walls_list()
{
	GLfloat emit[] = { 0.0, 1.0, 0.0, 1.0 };
	GLfloat specular[] = { 0.0, 1.0, 0.0, 1.0 };
	GLfloat mat_diffuse[] = { 0.5, 1.0, 0.5, 1.0 };
	GLfloat i, j;
	
	dl->walls_list = glGenLists(1);
	if (dl->walls_list != 0)
	{
		glNewList(dl->walls_list, GL_COMPILE);
		
		glPushMatrix();

		glCallList(dl->blue_list);

		glEnable(GL_TEXTURE_2D);

		glBindTexture(GL_TEXTURE_2D, tex->tex_names[METAL]);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);

		// left
		glRotated(90.0, 0.0, 1.0, 0.0);

		for (i = 30.0; i > -30.0; i -= 1.0)
			for (j = 0.0; j < 2.0; j += 1.0)
				draw_xy_tile(i, j, 23.0);

		for (i = 30.0; i > -30.0; i -= 1.0)
			for (j = 3.0; j < 13.0; j += 1.0)
				draw_xy_tile(i, j, 23.0);

		glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, mat_diffuse);
		glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, emit);
		glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specular);

		for (i = 30.0; i > -13.0; i -= 1.0)
			for (j = 2.0; j < 3.0; j += 1.0)
				draw_xy_tile(i, j, 23.0);

		for (i = -13.0; i > -21.0; i -= 1.0)
			for (j = 2.0; j < 3.0; j += 1.0)
				draw_xy_tile(i, j, 20.0);
		
		for (i = -21.0; i > -30.0; i -= 1.0)
			for (j = 2.0; j < 3.0; j += 1.0)
				draw_xy_tile(i, j, 23.0);

		// right
		glRotated(180.0, 0.0, 1.0, 0.0);

		glCallList(dl->blue_list);

		for (i = 30.0; i > -30.0; i -= 1.0)
			for (j = 0.0; j < 2.0; j += 1.0)
				draw_xy_tile(i, j, 23.0);

		for (i = 30.0; i > -30.0; i -= 1.0)
			for (j = 3.0; j < 13.0; j += 1.0)
				draw_xy_tile(i, j, 23.0);

		glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, mat_diffuse);
		glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, emit);
		glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specular);

		for (i = 30.0; i > 21.0; i -= 1.0)
			for (j = 2.0; j < 3.0; j += 1.0)
				draw_xy_tile(i, j, 23.0);
		
		for (i = 21.0; i > 13.0; i -= 1.0)
			for (j = 2.0; j < 3.0; j += 1.0)
				draw_xy_tile(i, j, 20.0);
		
		for (i = 13.0; i > -30.0; i -= 1.0)
			for (j = 2.0; j < 3.0; j += 1.0)
				draw_xy_tile(i, j, 23.0);

		// back
		glRotated(-90.0, 0.0, 1.0, 0.0);

		glCallList(dl->blue_list);

		for (i = 23.0; i > -23.0; i -= 1.0)
			for (j = 0.0; j < 2.0; j += 1.0)
				draw_xy_tile(i, j, 30.0);

		for (i = 23.0; i > -23.0; i -= 1.0)
			for (j = 3.0; j < 13.0; j += 1.0)
				draw_xy_tile(i, j, 30.0);

		glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, mat_diffuse);
		glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, emit);
		glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specular);

		for (i = 23.0; i > -23.0; i -= 1.0)
			for (j = 2.0; j < 3.0; j += 1.0)
				draw_xy_tile(i, j, 30.0);

		for (i = 23.0; i > 20.0; i -= 1.0)
			for (j = 2.0; j < 3.0; j += 1.0)
				draw_xy_tile(i, j, -21.0);
		
		for (i = -20.0; i > -23.0; i -= 1.0)
			for (j = 2.0; j < 3.0; j += 1.0)
				draw_xy_tile(i, j, -21.0);
		
		// front
		glRotated(180.0, 0.0, 1.0, 0.0);

		for (i = 23.0; i > 12.0; i -= 1.0)
			for (j = 2.0; j < 3.0; j += 1.0)
				draw_xy_tile(i, j, 30.0);

		for (i = -17.0; i > -23.0; i -= 1.0)
			for (j = 2.0; j < 3.0; j += 1.0)
				draw_xy_tile(i, j, 30.0);

		for (i = -20.0; i > -23.0; i -= 1.0)
			for (j = 2.0; j < 3.0; j += 1.0)
				draw_xy_tile(i, j, 13.0);
		
		for (i = 23.0; i > 20.0; i -= 1.0)
			for (j = 2.0; j < 3.0; j += 1.0)
				draw_xy_tile(i, j, 13.0);
		
		for (j = 0.0; j < 3.0; j += 1.0)
		{
			draw_xy_tile(12.0, j, 30.0);
			draw_xy_tile(-17.0, j, 30.0);
		}

		glDisable(GL_TEXTURE_2D);
		glCallList(dl->white_list);

		glPopMatrix();
		glEndList();
	}
	else
	{
		fprintf(stderr, "Unable to initialize walls display list, aborting\n");
		exit(1);
	}
}

/* Sets window display list */
void set_window_list()
{
	GLfloat emit[] = { 0.0, 0.0, 1.0, 1.0 };
	GLfloat specular[] = { 0.0, 0.0, 1.0, 0.3 };
	GLfloat mat_diffuse[] = { 0.3, 0.3, 0.8, 0.3 };
	GLfloat i, j;
	
	dl->window_list = glGenLists(1);
	if (dl->window_list != 0)
	{
		glNewList(dl->window_list, GL_COMPILE);
		glPushMatrix();

		glCallList(dl->blue_list);

		glEnable(GL_TEXTURE_2D);
		glBindTexture(GL_TEXTURE_2D, tex->tex_names[METAL]);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);

		// bottom
		for (i = 9.0; i > -15.0; i -= 1.0)
			for (j = 0.0; j < 2.0; j += 1.0)
				draw_xy_tile(i, j, 30.0);

		// top
		for (i = 9.0; i > -15.0; i -= 1.0)
			for (j = 10.0; j < 13.0; j += 1.0)
				draw_xy_tile(i, j, 30.0);

		// left
		for (i = 23.0; i > 12.0; i -= 1.0)
			for (j = 0.0; j < 2.0; j += 1.0)
				draw_xy_tile(i, j, 30.0);

		for (i = 11.0; i > 9.0; i -= 1.0)
			for (j = 0.0; j < 3.0; j += 1.0)
				draw_xy_tile(i, j, 30.0);

		for (i = 23.0; i > 9.0; i -= 1.0)
			for (j = 3.0; j < 13.0; j += 1.0)
				draw_xy_tile(i, j, 30.0);

		// right
		for (i = -15.0; i > -17.0; i -= 1.0)
			for (j = 0.0; j < 13.0; j += 1.0)
				draw_xy_tile(i, j, 30.0);

		for (i = -17.0; i > -23.0; i -= 1.0)
			for (j = 3.0; j < 13.0; j += 1.0)
				draw_xy_tile(i, j, 30.0);

		for (i = -18.0; i > -23.0; i -= 1.0)
			for (j = 0.0; j < 2.0; j += 1.0)
				draw_xy_tile(i, j, 30.0);

		glBindTexture(GL_TEXTURE_2D, tex->tex_names[DARK_METAL]);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		glCallList(dl->white_list);
		
		// bottom border
		for (i = 7.0; i > -13.0; i -= 1.0)
			for (j = 32.0; j > 30.0; j -= 1.0)
				draw_xz_tile(i, 2.0, j);

		glFrontFace(GL_CCW);

		// top border
		for (i = 7.0; i > -13.0; i -= 1.0)
			for (j = 32.0; j > 30.0; j -= 1.0)
				draw_xz_tile(i, 10.0, j);

		glFrontFace(GL_CW);

		glBegin(GL_QUADS);
		// bottom left border
		glNormal3f(-0.707, 0.707, 0.0);
		glTexCoord2f(0.0, 0.0); glVertex3f(7.0, 2.0, 30.0);
		glTexCoord2f(0.0, 1.0); glVertex3f(9.0, 4.0, 30.0);
		glTexCoord2f(1.0, 1.0); glVertex3f(9.0, 4.0, 32.0);
		glTexCoord2f(1.0, 0.0); glVertex3f(7.0, 2.0, 32.0);

		// top left border
		glNormal3f(-0.707, -0.707, 0.0);
		glTexCoord2f(0.0, 0.0); glVertex3f(9.0, 8.0, 30.0);
		glTexCoord2f(0.0, 1.0); glVertex3f(7.0, 10.0, 30.0);
		glTexCoord2f(1.0, 1.0); glVertex3f(7.0, 10.0, 32.0);
		glTexCoord2f(1.0, 0.0); glVertex3f(9.0, 8.0, 32.0);

		// top right border
		glNormal3f(0.707, -0.707, 0.0);
		glTexCoord2f(0.0, 0.0); glVertex3f(-15.0, 8.0, 32.0);
		glTexCoord2f(0.0, 1.0); glVertex3f(-13.0, 10.0, 32.0);
		glTexCoord2f(1.0, 1.0); glVertex3f(-13.0, 10.0, 30.0);
		glTexCoord2f(1.0, 0.0); glVertex3f(-15.0, 8.0, 30.0);

		// bottom right border
		glNormal3f(0.707, 0.707, 0.0);
		glTexCoord2f(0.0, 0.0); glVertex3f(-13.0, 2.0, 32.0);
		glTexCoord2f(0.0, 1.0); glVertex3f(-15.0, 4.0, 32.0);
		glTexCoord2f(1.0, 1.0); glVertex3f(-15.0, 4.0, 30.0);
		glTexCoord2f(1.0, 0.0); glVertex3f(-13.0, 2.0, 30.0);

		// left and right border
		for (i = 30.0; i < 32.0; i += 1.0)
			for (j = 4.0; j < 8.0; j += 1.0)
			{
				glNormal3f(-1.0, 0.0, 0.0);
				glTexCoord2f(0.0, 0.0); glVertex3f(9.0, j, i);
				glTexCoord2f(0.0, 1.0); glVertex3f(9.0, j+1.0, i);
				glTexCoord2f(1.0, 1.0); glVertex3f(9.0, j+1.0, i+1.0);
				glTexCoord2f(1.0, 0.0); glVertex3f(9.0, j, i+1.0);

				glNormal3f(1.0, 0.0, 0.0);
				glTexCoord2f(0.0, 0.0); glVertex3f(-15.0, j, i+1.0);
				glTexCoord2f(0.0, 1.0); glVertex3f(-15.0, j+1.0, i+1.0);
				glTexCoord2f(1.0, 1.0); glVertex3f(-15.0, j+1.0, i);
				glTexCoord2f(1.0, 0.0); glVertex3f(-15.0, j, i);
			}

		glEnd();

		glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, mat_diffuse);
		glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, emit);
		glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specular);
		
		glDisable(GL_TEXTURE_2D);
		
		// window
		for (i = 9.0; i > -15.0; i -= 1.0)
			for (j = 2.0; j < 10.0; j += 1.0)
				draw_xy_tile(i, j, 31.0);

		glEnable(GL_TEXTURE_2D);
		glBindTexture(GL_TEXTURE_2D, tex->tex_names[METAL]);

		glCallList(dl->blue_list);

		// wall triangles
		glBegin(GL_TRIANGLES);

		glNormal3f(0.0, 0.0, -1.0);
		// bottom left
		glTexCoord2f(0.0, 0.0); glVertex3f(9.0, 2.0, 30.0);
		glTexCoord2f(0.0, 1.0); glVertex3f(9.0, 4.0, 30.0);
		glTexCoord2f(1.0, 0.0); glVertex3f(7.0, 2.0, 30.0);

		// top left
		glTexCoord2f(0.0, 0.0); glVertex3f(9.0, 8.0, 30.0);
		glTexCoord2f(0.0, 1.0); glVertex3f(9.0, 10.0, 30.0);
		glTexCoord2f(1.0, 1.0); glVertex3f(7.0, 10.0, 30.0);

		// top right
		glTexCoord2f(0.0, 1.0); glVertex3f(-13.0, 10.0, 30.0);
		glTexCoord2f(1.0, 1.0); glVertex3f(-15.0, 10.0, 30.0);
		glTexCoord2f(1.0, 0.0); glVertex3f(-15.0, 8.0, 30.0);

		// bottom right
		glTexCoord2f(0.0, 0.0); glVertex3f(-13.0, 2.0, 30.0);
		glTexCoord2f(0.0, 1.0); glVertex3f(-15.0, 4.0, 30.0);
		glTexCoord2f(1.0, 0.0); glVertex3f(-15.0, 2.0, 30.0);

		glEnd();

		glPopMatrix();

		glDisable(GL_TEXTURE_2D);
		glCallList(dl->white_list);
		glEndList();
	}
	else
	{
		fprintf(stderr, "Unable to initialize window display list, aborting\n");
		exit(1);
	}
}

/* Sets ceiling display list */
void set_ceiling_list()
{
	GLfloat i, j, k;
	
	dl->ceiling_list = glGenLists(1);
	if (dl->ceiling_list != 0)
	{
		glNewList(dl->ceiling_list, GL_COMPILE);
		
		glPushMatrix();

		glCallList(dl->blue_list);
		//glCallList(dl->white_list);

		glEnable(GL_TEXTURE_2D);

		glBindTexture(GL_TEXTURE_2D, tex->tex_names[METAL]);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);

		glFrontFace(GL_CCW);
		// top
		for (i = 20.0; i > -20.0; i -= 1.0)
			for (j = 27.0; j > -27.0; j -= 1.0)
				draw_xz_tile(i, 16.0, j);
		glFrontFace(GL_CW);

		// left
		k = 13.0;
		for (i = 23.0; i > 20.0; i -= 1.0)
		{
			for (j = 27.0; j > -27.0; j -= 1.0)
			{
				glBegin(GL_QUADS);
				glNormal3f(-0.707, -0.707, 0.0);
				glTexCoord2f(0.0, 0.0); glVertex3f(i, k, j-1.0);
				glTexCoord2f(0.0, 1.0); glVertex3f(i-1.0, k+1.0, j-1.0);
				glTexCoord2f(1.0, 1.0); glVertex3f(i-1.0, k+1.0, j);
				glTexCoord2f(1.0, 0.0); glVertex3f(i, k, j);
				glEnd();
			}
			k++;
		}

		// right
		k = 13.0;
		for (i = -23.0; i < -20.0; i += 1.0)
		{
			for (j = 27.0; j > -27.0; j -= 1.0)
			{
				glBegin(GL_QUADS);
				glNormal3f(0.707, -0.707, 0.0);
				glTexCoord2f(0.0, 0.0); glVertex3f(i, k, j);
				glTexCoord2f(0.0, 1.0); glVertex3f(i+1.0, k+1.0, j);
				glTexCoord2f(1.0, 1.0); glVertex3f(i+1.0, k+1.0, j-1.0);
				glTexCoord2f(1.0, 0.0); glVertex3f(i, k, j-1.0);
				glEnd();
			}
			k++;
		}
		
		// front
		k = 13.0;
		for (j = 30.0; j > 27.0; j -= 1.0)
		{
			for (i = 20.0; i > -20.0; i -= 1.0)
			{
				glBegin(GL_QUADS);
				glNormal3f(0.0, -0.707, -0.707);
				glTexCoord2f(0.0, 0.0); glVertex3f(i, k, j);
				glTexCoord2f(0.0, 1.0); glVertex3f(i, k+1.0, j-1.0);
				glTexCoord2f(1.0, 1.0); glVertex3f(i-1.0, k+1.0, j-1.0);
				glTexCoord2f(1.0, 0.0); glVertex3f(i-1.0, k, j);
				glEnd();
			}
			k++;
		}
		
		// back
		k = 13.0;
		for (j = -30.0; j < -27.0; j += 1.0)
		{
			for (i = -20.0; i < 20.0; i += 1.0)
			{
				glBegin(GL_QUADS);
				glNormal3f(0.0, -0.707, 0.707);
				glTexCoord2f(0.0, 0.0); glVertex3f(i, k, j);
				glTexCoord2f(0.0, 1.0); glVertex3f(i, k+1.0, j+1.0);
				glTexCoord2f(1.0, 1.0); glVertex3f(i+1.0, k+1.0, j+1.0);
				glTexCoord2f(1.0, 0.0); glVertex3f(i+1.0, k, j);
				glEnd();
			}
			k++;
		}
		
		// corners
		glBegin(GL_QUADS);
		glNormal3f(-0.577, -0.577, -0.577);
		glTexCoord2f(0.0, 0.0); glVertex3f(23.0, 13.0, 27.0);
		glTexCoord2f(0.0, 1.0); glVertex3f(20.0, 16.0, 27.0);
		glTexCoord2f(1.0, 1.0); glVertex3f(20.0, 13.0, 30.0);
		glTexCoord2f(1.0, 0.0); glVertex3f(23.0, 13.0, 30.0);

		glNormal3f(0.577, -0.577, -0.577);
		glTexCoord2f(0.0, 0.0); glVertex3f(-20.0, 13.0, 30.0);
		glTexCoord2f(0.0, 1.0); glVertex3f(-20.0, 16.0, 27.0);
		glTexCoord2f(1.0, 1.0); glVertex3f(-23.0, 13.0, 27.0);
		glTexCoord2f(1.0, 0.0); glVertex3f(-23.0, 13.0, 30.0);

		glNormal3f(0.577, -0.577, 0.577);
		glTexCoord2f(0.0, 0.0); glVertex3f(-23.0, 13.0, -27.0);
		glTexCoord2f(0.0, 1.0); glVertex3f(-20.0, 16.0, -27.0);
		glTexCoord2f(1.0, 1.0); glVertex3f(-20.0, 13.0, -30.0);
		glTexCoord2f(1.0, 0.0); glVertex3f(-23.0, 13.0, -30.0);

		glNormal3f(-0.577, -0.577, 0.577);
		glTexCoord2f(0.0, 0.0); glVertex3f(20.0, 13.0, -30.0);
		glTexCoord2f(0.0, 1.0); glVertex3f(20.0, 16.0, -27.0);
		glTexCoord2f(1.0, 1.0); glVertex3f(23.0, 13.0, -27.0);
		glTexCoord2f(1.0, 0.0); glVertex3f(23.0, 13.0, -30.0);
		glEnd();
		
		glDisable(GL_TEXTURE_2D);
		
		glCallList(dl->white_list);
		glPopMatrix();
		glEndList();
	}
	else
	{
		fprintf(stderr, "Unable to initialize ceiling display list, aborting\n");
		exit(1);
	}
}

/* Sets floor display list */
void set_floor_list()
{
	GLfloat i, j;
	
	dl->floor_list = glGenLists(1);
	if (dl->floor_list != 0)
	{
		glNewList(dl->floor_list, GL_COMPILE);
		
		//glCallList(dl->white_list);
		glEnable(GL_TEXTURE_2D);
		// metal floor
		glBindTexture(GL_TEXTURE_2D, tex->tex_names[FLOOR_METAL]);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		for (i = 23.0; i > -23.0; i -= 1.0)
			for (j = 30.0; j > -10.0; j -= 1.0)
				draw_xz_tile(i, 0.0, j);

		for (i = 23.0; i > -11.0; i -= 1.0)
			for (j = -10.0; j > -30.0; j -= 1.0)
				draw_xz_tile(i, 0.0, j);

		// floor gratings
		glBindTexture(GL_TEXTURE_2D, tex->tex_names[FLOOR_GRATE]);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);

		// lower left
		//for (i = 18.0; i > 10.0; i -= 1.0)
		//	for (j = -16.0; j > -25.0; j -= 1.0)
		//		draw_xz_tile(i, 0.0, j);

		// lower right
		for (i = -11.0; i > -23.0; i -= 1.0)
			for (j = -10.0; j > -30.0; j -= 1.0)
				draw_xz_tile(i, 0.0, j);

		// walkways
		glBindTexture(GL_TEXTURE_2D, tex->tex_names[BUMP_PLAT]);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);

		// upper along X
		for (i = 23.0; i > -17.0; i -= 1.0)
			for (j = 8.0; j > 6.0; j -= 1.0)
				draw_xz_tile(i, 0.05, j);

		// left along Z
		for (i = 6.0; i > 4.0; i -= 1.0)
			for (j = 11.0; j > -12.0; j -= 1.0)
				draw_xz_tile(i, 0.05, j);

		// lower along X
		for (i = 23.0; i > 5.0; i -= 1.0)
			for (j = -11.0; j > -13.0; j -= 1.0)
				draw_xz_tile(i, 0.05, j);

		// right along Z
		for (i = -16.0; i > -18.0; i -= 1.0)
			for (j = 7.0; j > -2.0; j -= 1.0)
				draw_xz_tile(i, 0.05, j);

		// right along X
		for (i = -17.0; i > -23.0; i -= 1.0)
			for (j = -1.0; j > -3.0; j -= 1.0)
				draw_xz_tile(i, 0.05, j);

		// walkway triangles
		glBegin(GL_TRIANGLES);

		glNormal3f(0.0, 1.0, 0.0);
		glTexCoord2f(0.0, 0.0); glVertex3f(5.0, 0.05, -13.0);
		glTexCoord2f(0.0, 1.0); glVertex3f(5.0, 0.05, -12.0);
		glTexCoord2f(1.0, 1.0); glVertex3f(4.0, 0.05, -12.0);

		glTexCoord2f(0.0, 0.0); glVertex3f(-17.0, 0.05, 7.0);
		glTexCoord2f(0.0, 1.0); glVertex3f(-17.0, 0.05, 8.0);
		glTexCoord2f(1.0, 0.0); glVertex3f(-18.0, 0.05, 7.0);

		glTexCoord2f(0.0, 1.0); glVertex3f(-16.0, 0.05, -2.0);
		glTexCoord2f(1.0, 1.0); glVertex3f(-17.0, 0.05, -2.0);
		glTexCoord2f(1.0, 0.0); glVertex3f(-17.0, 0.05, -3.0);

		// central triangles
		glTexCoord2f(0.0, 0.0); glVertex3f(4.0, 0.05, 8.0);
		glTexCoord2f(0.0, 1.0); glVertex3f(4.0, 0.05, 9.0);
		glTexCoord2f(1.0, 0.0); glVertex3f(3.0, 0.05, 8.0);

		glTexCoord2f(0.0, 0.0); glVertex3f(7.0, 0.05, 8.0);
		glTexCoord2f(1.0, 1.0); glVertex3f(6.0, 0.05, 9.0);
		glTexCoord2f(1.0, 0.0); glVertex3f(6.0, 0.05, 8.0);

		glTexCoord2f(0.0, 1.0); glVertex3f(7.0, 0.05, 6.0);
		glTexCoord2f(1.0, 1.0); glVertex3f(6.0, 0.05, 6.0);
		glTexCoord2f(1.0, 0.0); glVertex3f(6.0, 0.05, 5.0);

		glTexCoord2f(0.0, 0.0); glVertex3f(4.0, 0.05, 5.0);
		glTexCoord2f(0.0, 1.0); glVertex3f(4.0, 0.05, 6.0);
		glTexCoord2f(1.0, 1.0); glVertex3f(3.0, 0.05, 6.0);

		glEnd();
		glDisable(GL_TEXTURE_2D);

		glPushMatrix();
		glTranslated(0.0, 0.0, 20.0);
		glCallList(dl->redlight_list);
		glPopMatrix();
		glEndList();
	}
	else
	{
		fprintf(stderr, "Unable to initialize floor display list, aborting\n");
		exit(1);
	}
}

/* display list to draw monitor base */
void set_monitor_base_list()
{
	dl->monitor_base_list = glGenLists(1);
	if (dl->monitor_base_list != 0)
	{
		glNewList(dl->monitor_base_list, GL_COMPILE);

		glBindTexture(GL_TEXTURE_2D, tex->tex_names[METAL]);
		
		glBegin(GL_QUADS);
		// front
		glNormal3f(0.0, 0.0, -1.0);
		glTexCoord2f(0.0, 0.0); glVertex3f(0.05, -0.8, -1.25);
		glTexCoord2f(0.0, 1.0); glVertex3f(0.05, 0.8, -1.25);
		glTexCoord2f(1.0, 1.0); glVertex3f(-0.05, 0.8, -1.25);
		glTexCoord2f(1.0, 0.0); glVertex3f(-0.05, -0.8, -1.25);

		// top
		glNormal3f(0.0, 1.0, 0.0);
		glTexCoord2f(0.0, 0.0); glVertex3f(0.05, 0.8, -1.25);
		glTexCoord2f(0.0, 1.0); glVertex3f(0.05, 0.8, 1.25);
		glTexCoord2f(1.0, 1.0); glVertex3f(-0.05, 0.8, 1.25);
		glTexCoord2f(1.0, 0.0); glVertex3f(-0.05, 0.8, -1.25);

		// back
		glNormal3f(0.0, 0.0, 1.0);
		glTexCoord2f(0.0, 0.0); glVertex3f(-0.05, -0.8, 1.25);
		glTexCoord2f(0.0, 1.0); glVertex3f(-0.05, 0.8, 1.25);
		glTexCoord2f(1.0, 1.0); glVertex3f(0.05, 0.8, 1.25);
		glTexCoord2f(1.0, 0.0); glVertex3f(0.05, -0.8, 1.25);

		// bottom
		glNormal3f(0.0, -1.0, 0.0);
		glTexCoord2f(0.0, 0.0); glVertex3f(-0.05, -0.8, 1.25);
		glTexCoord2f(0.0, 1.0); glVertex3f(-0.05, -0.8, -1.25);
		glTexCoord2f(1.0, 1.0); glVertex3f(0.05, -0.8, -1.25);
		glTexCoord2f(1.0, 0.0); glVertex3f(0.05, -0.8, 1.25);
		glEnd();

		glEndList();
	}
	else
	{
		fprintf(stderr, "Unable to initialize monitor_base display list, aborting\n");
		exit(1);
	}
}

/* display list to draw a monitor */
void set_monitor_list()
{
	GLfloat i, j, k;
	
	dl->monitor_list = glGenLists(1);
	if (dl->monitor_list != 0)
	{
		glNewList(dl->monitor_list, GL_COMPILE);

		glPushMatrix();
		glEnable(GL_TEXTURE_2D);
		glBindTexture(GL_TEXTURE_2D, tex->tex_names[METAL]);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);

		glCallList(dl->blue_list);

		glBegin(GL_QUADS);

		// walls
		// bottom
		glNormal3f(1.0, 0.0, 0.0);
		for (i = 0.0; i < 2.0; i += 1.0)
			for (j = 21.0; j > 13.0; j -= 1.0)
			{
				glTexCoord2f(0.0, 0.0); glVertex3f(-20.0, i, j);
				glTexCoord2f(0.0, 1.0); glVertex3f(-20.0, i+1.0, j);
				glTexCoord2f(1.0, 1.0); glVertex3f(-20.0, i+1.0, j-1.0);
				glTexCoord2f(1.0, 0.0); glVertex3f(-20.0, i, j-1.0);
			}

		// top
		for (i = 3.0; i < 4.0; i += 1.0)
			for (j = 21.0; j > 13.0; j -= 1.0)
			{
				glTexCoord2f(0.0, 0.0); glVertex3f(-20.0, i, j);
				glTexCoord2f(0.0, 1.0); glVertex3f(-20.0, i+1.0, j);
				glTexCoord2f(1.0, 1.0); glVertex3f(-20.0, i+1.0, j-1.0);
				glTexCoord2f(1.0, 0.0); glVertex3f(-20.0, i, j-1.0);
			}
		
		// slanted top
		k = 4.0;
		glNormal3f(0.707, 0.707, 0.0);
		for (i = -20.0; i > -23.0; i -= 1.0)
		{
			for (j = 21.0; j > 13.0; j -= 1.0)
			{
				glTexCoord2f(0.0, 0.0); glVertex3f(i, k, j);
				glTexCoord2f(0.0, 1.0); glVertex3f(i-1.0, k+1.0, j);
				glTexCoord2f(1.0, 1.0); glVertex3f(i-1.0, k+1.0, j-1.0);
				glTexCoord2f(1.0, 0.0); glVertex3f(i, k, j-1.0);
			}
			k++;
		}
		
		glEnd();

		// back bottom
		for (i = -20.0; i > -23.0; i -= 1.0)
			for (j = 0.0; j < 2.0; j += 1.0)
				draw_xy_tile(i, j, 13.0);

		// back top
		for (i = -20.0; i > -23.0; i -= 1.0)
			for (j = 3.0; j < 4.0; j += 1.0)
				draw_xy_tile(i, j, 13.0);

		draw_xy_tile(-21.0, 4.0, 13.0);
		draw_xy_tile(-22.0, 4.0, 13.0);
		draw_xy_tile(-22.0, 5.0, 13.0);
		glFrontFace(GL_CCW);
		draw_xy_tile(-21.0, 4.0, 21.0);
		draw_xy_tile(-22.0, 4.0, 21.0);
		draw_xy_tile(-22.0, 5.0, 21.0);
		glFrontFace(GL_CW);
		
		i = -20.0;
		for (j = 4.0; j < 7.0 ; j += 1.0)
		{
			glBegin(GL_TRIANGLES);
			glNormal3f(0.0, 0.0, -1.0);
			glTexCoord2f(0.0, 0.0); glVertex3f(i, j, 13.0);
			glTexCoord2f(1.0, 1.0); glVertex3f(i-1.0, j+1.0, 13.0);
			glTexCoord2f(1.0, 0.0); glVertex3f(i-1.0, j, 13.0);
			glEnd();

			glFrontFace(GL_CCW);
			glBegin(GL_TRIANGLES);
			glNormal3f(0.0, 0.0, 1.0);
			glTexCoord2f(0.0, 0.0); glVertex3f(i, j, 21.0);
			glTexCoord2f(1.0, 1.0); glVertex3f(i-1.0, j+1.0, 21.0);
			glTexCoord2f(1.0, 0.0); glVertex3f(i-1.0, j, 21.0);
			glEnd();
			glFrontFace(GL_CW);

			i -= 1.0;
		}

		// front bottom
		glFrontFace(GL_CCW);
		for (i = -20.0; i > -23.0; i -= 1.0)
			for (j = 0.0; j < 2.0; j += 1.0)
				draw_xy_tile(i, j, 21.0);

		// front top
		for (i = -20.0; i > -23.0; i -= 1.0)
			for (j = 3.0; j < 4.0; j += 1.0)
				draw_xy_tile(i, j, 21.0);
		glFrontFace(GL_CW);

		glCallList(dl->white_list);

		glTranslated(-19.95, 2.5, 19.0);
		draw_monitor(MONITOR1);
		glTranslated(0.0, 0.0, -4.25);
		draw_monitor(MONITOR2);

		glDisable(GL_TEXTURE_2D);
		glPopMatrix();
		glEndList();
	}
	else
	{
		fprintf(stderr, "Unable to initialize monitor display list, aborting\n");
		exit(1);
	}
}

/* display list to draw lights */
void set_lights_list()
{
	GLfloat i;
	
	dl->lights_list = glGenLists(1);
	if (dl->lights_list != 0)
	{
		glNewList(dl->lights_list, GL_COMPILE);

		glFrontFace(GL_CCW);
		glBegin(GL_QUADS);
		glNormal3f(0.0, -1.0, 0.0);
		
		if (state->hi_res)
		{
			for (i = -5.0; i < 5.0; i += 0.5)
			{
				glVertex3f(i, 0.0, 0.5);
				glVertex3f(i, 0.0, 0.0);
				glVertex3f(i+0.5, 0.0, 0.0);
				glVertex3f(i+0.5, 0.0, 0.5);

				glVertex3f(i, 0.0, 0.0);
				glVertex3f(i, 0.0, -0.5);
				glVertex3f(i+0.5, 0.0, -0.5);
				glVertex3f(i+0.5, 0.0, 0.0);
			}	
		}
		else
		{
			for (i = -5.0; i < 5.0; i += 1.0)
			{
				glVertex3f(i, 0.0, 0.5);
				glVertex3f(i, 0.0, -0.5);
				glVertex3f(i+1.0, 0.0, -0.5);
				glVertex3f(i+1.0, 0.0, 0.5);
			}
		}
		
		glEnd();
		
		glEnable(GL_TEXTURE_2D);
		glBindTexture(GL_TEXTURE_2D, tex->tex_names[METAL]);
		
		// light frame
		glBegin(GL_QUADS);
		glTexCoord2f(0.0, 0.0); glVertex3f(-5.2, 0.1, 0.7);
		glTexCoord2f(0.0, 1.0); glVertex3f(-5.2, 0.1, -0.7);
		glTexCoord2f(1.0, 1.0); glVertex3f(-5.0, 0.0, -0.5);
		glTexCoord2f(1.0, 0.0); glVertex3f(-5.0, 0.0, 0.5);

		glTexCoord2f(0.0, 0.0); glVertex3f(-5.2, 0.1, -0.7);
		glTexCoord2f(0.0, 1.0); glVertex3f(5.2, 0.1, -0.7);
		glTexCoord2f(1.0, 1.0); glVertex3f(5.0, 0.0, -0.5);
		glTexCoord2f(1.0, 0.0); glVertex3f(-5.0, 0.0, -0.5);

		glTexCoord2f(0.0, 0.0); glVertex3f(5.2, 0.1, -0.7);
		glTexCoord2f(0.0, 1.0); glVertex3f(5.2, 0.1, 0.7);
		glTexCoord2f(1.0, 1.0); glVertex3f(5.0, 0.0, 0.5);
		glTexCoord2f(1.0, 0.0); glVertex3f(5.0, 0.0, -0.5);

		glTexCoord2f(0.0, 0.0); glVertex3f(5.2, 0.1, 0.7);
		glTexCoord2f(0.0, 1.0); glVertex3f(-5.2, 0.1, 0.7);
		glTexCoord2f(1.0, 1.0); glVertex3f(-5.0, 0.0, 0.5);
		glTexCoord2f(1.0, 0.0); glVertex3f(5.0, 0.0, 0.5);
		glEnd();

		glDisable(GL_TEXTURE_2D);
		glFrontFace(GL_CW);
		glEndList();
	}
	else
	{
		fprintf(stderr, "Unable to initialize lights display list, aborting\n");
		exit(1);
	}
}

/* display list to draw platforms */
void set_platform_list()
{
	dl->platform_list = glGenLists(1);
	if (dl->platform_list != 0)
	{
		glNewList(dl->platform_list, GL_COMPILE);

		glBindTexture(GL_TEXTURE_2D, tex->tex_names[WARN]);
		
		glBegin(GL_QUADS);
		glNormal3f(0.0, 1.0, 0.0);
		// right
		glTexCoord2f(0.0, 1.0); glVertex3f(-2.0, 0.1, -2.0);
		glTexCoord2f(1.0, 1.0); glVertex3f(-2.0, 0.1, 2.0);
		glTexCoord2f(1.0, 0.0); glVertex3f(-2.3, 0.0, 2.3);
		glTexCoord2f(0.0, 0.0); glVertex3f(-2.3, 0.0, -2.3);

		// back
		glTexCoord2f(0.0, 0.0); glVertex3f(2.3, 0.0, -2.3);
		glTexCoord2f(0.0, 1.0); glVertex3f(2.0, 0.1, -2.0);
		glTexCoord2f(1.0, 1.0); glVertex3f(-2.0, 0.1, -2.0);
		glTexCoord2f(1.0, 0.0); glVertex3f(-2.3, 0.0, -2.3);

		// left
		glTexCoord2f(1.0, 0.0); glVertex3f(2.3, 0.0, -2.3);
		glTexCoord2f(0.0, 0.0); glVertex3f(2.3, 0.0, 2.3);
		glTexCoord2f(0.0, 1.0); glVertex3f(2.0, 0.1, 2.0);
		glTexCoord2f(1.0, 1.0); glVertex3f(2.0, 0.1, -2.0);

		// top
		glTexCoord2f(1.0, 1.0); glVertex3f(2.0, 0.1, 2.0);
		glTexCoord2f(1.0, 0.0); glVertex3f(2.3, 0.0, 2.3);
		glTexCoord2f(0.0, 0.0); glVertex3f(-2.3, 0.0, 2.3);
		glTexCoord2f(0.0, 1.0); glVertex3f(-2.0, 0.1, 2.0);
		glEnd();
		
		glEndList();
	}
	else
	{
		fprintf(stderr, "Unable to initialize platform display list, aborting\n");
		exit(1);
	}
}

/* display list to draw ventilation items */
void set_ventilation_list()
{
	int i;

	dl->ventilation_list = glGenLists(1);
	if (dl->ventilation_list != 0)
	{
		glNewList(dl->ventilation_list, GL_COMPILE);

		// left wall decor
		glPushMatrix();

		glTranslated(23.0, 6.0, 7.0);
		glRotated(90.0, 0.0, 1.0, 0.0);
		glCallList(dl->fan_list);

		glPushMatrix();
		glTranslated(9.5, 0.0, 0.0);

		glPushMatrix();
		glTranslated(0.0, -3.5, 0.0);
		glRotated(90.0, 0.0, 1.0, 0.0);
		draw_monitor(MONITOR3);
		glTranslated(28.0, 0.0, 27.5);
		glRotated(90.0, 0.0, 1.0, 0.0);
		draw_monitor(MONITOR4);
		glPopMatrix();

		glRotated(180.0, 0.0, 1.0, 0.0);
		glCallList(dl->fan_list);
		glPopMatrix();

		glPushMatrix();
		glTranslated(-0.5, 0.0, 0.0);
		glScaled(1.4, 1.4, 1.4);
		for (i = 0; i < 10; i++)
		{
			glTranslated(2.0, 0.0, 0.0);
			glCallList(dl->tunnel_list);
		}
		glRotated(-90.0, 0.0, 0.0, 1.0);
		glTranslated(0.5, 0.0, 0.0);
		for (i = 0; i < 3; i++)
		{
			glCallList(dl->tunnel_list);
			glTranslated(2.0, 0.0, 0.0);
		}
		glPopMatrix();

		glTranslated(19.0, 0.0, 0.0);
		glCallList(dl->fan_list);
		// panel
		glTranslated(-23.0, -4.0, -0.1);
		glCallList(dl->panel_list);
		glPopMatrix();

		glPushMatrix();
		// ceiling
		glTranslated(0.0, 15.0, -24.0);
		glRotated(90.0, 0.0, 1.0, 0.0);
		for (i = 0; i < 24; i++)
		{
			glCallList(dl->tunnel_list);
			glTranslated(-2.0, 0.0, 0.0);
		}
		// horizontal
		glRotated(-90.0, 0.0, 1.0, 0.0);
		glTranslated(17.0, 0.0, -15.75);

		glPushMatrix();
		glTranslated(0.0, 0.0, 0.0);
		glRotated(-90.0, 1.0, 0.0, 0.0);
		glCallList(dl->fan_list);
		glPopMatrix();

		for (i = 0; i < 16; i++)
		{
			glTranslated(-2.0, 0.0, 0.0);
			glCallList(dl->tunnel_list);
		}

		glPushMatrix();
		glTranslated(-2.0, 0.0, 0.0);
		glRotated(-90.0, 1.0, 0.0, 0.0);
		glCallList(dl->fan_list);
		glPopMatrix();

		glPushMatrix();
		glTranslated(15.0, 0.0, 0.0);
		glRotated(90.0, 1.0, 0.0, 0.0);
		glCallList(dl->fan_list);
		glPopMatrix();

		glTranslated(32.0, 0.0, -16.5);

		glPushMatrix();
		glTranslated(0.0, 0.0, 0.0);
		glRotated(-90.0, 1.0, 0.0, 0.0);
		glCallList(dl->fan_list);
		glPopMatrix();

		for (i = 0; i < 16; i++)
		{
			glTranslated(-2.0, 0.0, 0.0);
			glCallList(dl->tunnel_list);
		}

		glPushMatrix();
		glTranslated(-2.0, 0.0, 0.0);
		glRotated(-90.0, 1.0, 0.0, 0.0);
		glCallList(dl->fan_list);
		glPopMatrix();	

		glPushMatrix();
		glTranslated(15.0, 0.0, 0.0);
		glRotated(90.0, 1.0, 0.0, 0.0);
		glCallList(dl->fan_list);
		glPopMatrix();

		glPopMatrix();
		glEndList();
	}
	else
	{
		fprintf(stderr, "Unable to initialize ventilation display list, aborting\n");
		exit(1);
	}
}

/* display list to draw "decor" items */
void set_decor_list()
{
	GLfloat i;
	
	dl->decor_list = glGenLists(1);
	if (dl->decor_list != 0)
	{
		glNewList(dl->decor_list, GL_COMPILE);

		// platforms
		glPushMatrix();
		glTranslated(10.0, 0.0, 18.0);
		glScaled(1.5, 1.0, 1.5);
		draw_platform(METAL2);
		glTranslated(-13.5, 0.0, 0.0);
		draw_platform(METAL2);
		glTranslated(3.0, 0.0, -13.5);
		//draw_platform(DARK_METAL);
		glTranslated(12.0, 0.0, 0.0);
		draw_platform(DARK_METAL);
		glTranslated(1.0, 0.0, -12.0);
		glScaled(1.2, 1.2, 1.2);
		draw_platform(FLOOR_GRATE);
		glPopMatrix();
		
		glPushMatrix();
		glRotated(180.0, 0.0, 1.0, 0.0);
		glTranslated(0.0, 3.0, 30.0);
		glRotated(-90.0, 1.0, 0.0, 0.0);
		draw_platform(VENT);
		glPopMatrix();

		// lower left area decor
		// containers
		glPushMatrix();
		glPushMatrix();
		glTranslated(-18.0, 2.55, -23.0);
		glRotated(30, 0.0, 1.0, 0.0);
		glScaled(2.5, 2.5, 2.5);
		glCallList(dl->container_list);
		glPopMatrix();
		glTranslated(14.0, 1.6, -21.5);
		//glRotated(30, 0.0, 1.0, 0.0);
		glScaled(1.5, 1.5, 1.5);
		glCallList(dl->container_list);
		glTranslated(0.0, 1.5, 0.0);
		glRotated(20, 0.0, 1.0, 0.0);
		glScaled(0.5, 0.5, 0.5);
		glCallList(dl->fan_list);
		glPopMatrix();

		glPushMatrix();
		glTranslated(-21.0, 1.0, 2.0);
		glRotated(20, 0.0, 1.0, 0.0);
		glCallList(dl->container_list);
		glPushMatrix();
		glTranslated(0.0, 0.0, 3.0);
		glRotated(45, 0.0, 1.0, 0.0);
		glCallList(dl->container_list);
		glPopMatrix();
		glRotated(-20, 0.0, 1.0, 0.0);
		glTranslated(42.0, 0.5, 26.0);
		glScaled(1.5, 1.5, 1.5);
		glCallList(dl->container_list);
		glPopMatrix();

		// posts
		// around the fence
		// 4 posts on one side, 2 on the other
		glPushMatrix();
		for(i = -15.0; i >= -20.0; i-=5.0)
		{
			glPushMatrix();
			glTranslatef(i, 0.1, -9.15);
			glCallList(dl->double_post_list);
			glPopMatrix();
		}
		for(i = -14.0; i >= -30.0; i-=5.0)
		{
			glPushMatrix();
			glTranslatef(-9.85, 0.1, i);
			glCallList(dl->double_post_list);
			glPopMatrix();
		}
		
		// in the fenced area (posts lying down)
		glPushMatrix();
		glTranslated(-9.85, 0.1, -9.15);
		glCallList(dl->post_list);
		glTranslated(0.0, 2.0, 0.0);
		glCallList(dl->post_list);
		glTranslated(-4.0, -1.7, -15.0);
		glRotated(90.0, 0.0, 0.0, 1.0);
		glRotated(30.0, 1.0, 0.0, 0.0);
		glCallList(dl->post_list);
		glTranslated(0.0, 0.0, -2.0);
		glRotated(20.0, 1.0, 0.0, 0.0);
		glCallList(dl->post_list);
		glTranslated(0.0, -4.0, 5.0);
		glRotated(35.0, 1.0, 0.0, 0.0);
		glCallList(dl->post_list);
		// north side
		glTranslated(0.0, 15.0, -10.0);
		glCallList(dl->post_list);
		glTranslated(0.0, 0.0, 0.9);
		glCallList(dl->post_list);
		glTranslated(0.0, 0.0, 0.9);
		glCallList(dl->post_list);
		glPushMatrix();
		glRotated(90.0, 1.0, 0.0, 0.0);
		glTranslated(0.3, -1.8, -0.8);
		glCallList(dl->post_list);
		glTranslated(0.0, -0.1, -0.8);
		glCallList(dl->post_list);
		glPopMatrix();

		glRotated(35.0, 1.0, 0.0, 0.0);
		glTranslated(0.0, 0.0, 3.0);
		glCallList(dl->post_list);
		glTranslated(0.7, -5.5, -3.0);
		glCallList(dl->fan_list);
		glPopMatrix();
		
		// floor
		glPushMatrix();
		glTranslated(2.0, 0.3, -15.0);
		glRotated(90.0, 0.0, 0.0, 1.0);
		glRotated(15.0, 1.0, 0.0, 0.0);
		glCallList(dl->post_list);
		glTranslated(0.0, -3.0, -5.0);
		glRotated(20.0, 1.0, 0.0, 0.0);
		glCallList(dl->post_list);
		glPopMatrix();

		// console
		glPushMatrix();
		glTranslated(5.8, 0.0, 10.3);
		glCallList(dl->post_list);
		glTranslated(-1.3, 0.0, 0.0);
		glCallList(dl->post_list);
		glPopMatrix();
		glPopMatrix();

		glPushMatrix();
		// lower left door
		glTranslated(22.8, 0.0, -12.0);
		glRotated(90, 0.0, 1.0, 0.0);
		glCallList(dl->door_list);
		// upper left door
		glTranslated(-19.0, 0.0, 0.0);
		glCallList(dl->door_list);
		// right door
		glTranslated(9.0, 0.0, -45.6);
		glRotated(180, 0.0, 1.0, 0.0);
		glCallList(dl->door_list);
		glTranslated(1.0, 5.0, 0.0);
		glRotated(-90.0, 1.0, 0.0, 0.0);
		glCallList(dl->redlight_list);
		glTranslated(-2.0, 0.0, 0.0);
		glCallList(dl->redlight_list);
		glRotated(90.0, 1.0, 0.0, 0.0);
		glTranslated(-3.0, -3.0, 0.05);
		glCallList(dl->panel_list);
		glPopMatrix();

		glEndList();
	}
	else
	{
		fprintf(stderr, "Unable to initialize decor display list, aborting\n");
		exit(1);
	}
}
