/*------------------------------------------------------------------------------
* Alpha Shooter
* A 3D OpenGL First-Person Shooter game
* Copyright (C) 2005, 2006, 2007 Nicola Cocchiaro <ncocchiaro@users.sf.net>
* Alpha Shooter can be found at http://www.sf.net/projects/alphashooter.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 or, (at your option) any later
* version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*-------------------------------------------------------------------------------
*/

// lib.c
// some helper functions

#include <stdio.h>
#include <stdlib.h>
#include "lib.h"


/* Returns the absolute value of a float */
GLdouble float_abs(GLdouble f)
{
	if (f < 0.0)
		return (0.0-f);
	else
		return(f);
}

/* useful for finding the fovy value */
/*GLdouble calculateAngle(GLdouble size, GLdouble distance)
{
	GLdouble radtheta, degtheta;
	radtheta = 2.0 * atan2(size/2.0, distance);
	degtheta = (180.0 * radtheta) / M_PI;
	return(degtheta);
}
*/
/* adds a delta to an angle clipping it between -360 and 360 degrees */
GLdouble add_angle(GLdouble angle, GLdouble deg_delta)
{
	GLdouble res;
	if (deg_delta != 0.0)
	{
		res = angle + deg_delta;
		
		while (res > (360.0-ROTSTEP))
			res = 0.0;
		while (res < (-359.0+ROTSTEP))
			res = 0.0;
		return(res);
	}
	return(angle);
}

/* Initializes a matrix setting it to the identity matrix */
void identity_matrix(GLdouble m[16])
{
	m[0] = 1.0;
	m[1] = 0.0;
	m[2] = 0.0;
	m[3] = 0.0;
	m[4] = 0.0;
	m[5] = 1.0;
	m[6] = 0.0;
	m[7] = 0.0;
	m[8] = 0.0;
	m[9] = 0.0;
	m[10] = 1.0;
	m[11] = 0.0;
	m[12] = 0.0;
	m[13] = 0.0;
	m[14] = 0.0;
	m[15] = 1.0;
}
		
/* multiplies 4x4 matrix m by 4x4 matrix n */
void matrix_multiply(GLdouble m[16], GLdouble n[16], GLdouble r[16])
{
	r[0] = m[0]*n[0] + m[1]*n[4] + m[2]*n[8] + m[3]*n[12];
	r[1] = m[0]*n[1] + m[1]*n[5] + m[2]*n[9] + m[3]*n[13];
	r[2] = m[0]*n[2] + m[1]*n[6] + m[2]*n[10] + m[3]*n[14];
	r[3] = m[0]*n[3] + m[1]*n[7] + m[2]*n[11] + m[3]*n[15];
	
	r[4] = m[4]*n[0] + m[5]*n[4] + m[6]*n[8] + m[7]*n[12];
	r[5] = m[4]*n[1] + m[5]*n[5] + m[6]*n[9] + m[7]*n[13];
	r[6] = m[4]*n[2] + m[5]*n[6] + m[6]*n[10] + m[7]*n[14];
	r[7] = m[4]*n[3] + m[5]*n[7] + m[6]*n[11] + m[7]*n[15];

	r[8] = m[8]*n[0] + m[9]*n[4] + m[10]*n[8] + m[11]*n[12];
	r[9] = m[8]*n[1] + m[9]*n[5] + m[10]*n[9] + m[11]*n[13];
	r[10] = m[8]*n[2] + m[9]*n[6] + m[10]*n[10] + m[11]*n[14];
	r[11] = m[8]*n[3] + m[9]*n[7] + m[10]*n[11] + m[11]*n[15];
	
	r[12] = m[12]*n[0] + m[13]*n[4] + m[14]*n[8] + m[15]*n[12];
	r[13] = m[12]*n[1] + m[13]*n[5] + m[14]*n[9] + m[15]*n[13];
	r[14] = m[12]*n[2] + m[13]*n[6] + m[14]*n[10] + m[15]*n[14];
	r[15] = m[12]*n[3] + m[13]*n[7] + m[14]*n[11] + m[15]*n[15];
}

/* multiplies vector v by matrix m */
void apply_v2m(vector *v, GLdouble m[16])
{
	vector a;

	a.x = v->x;
	a.y = v->y;
	a.z = v->z;
	a.w = v->w;

	v->x = a.x*m[0] + a.y*m[1] + a.z*m[2]  + a.w*m[3];
	v->y = a.x*m[4] + a.y*m[5] + a.z*m[6]  + a.w*m[7];
	v->z = a.x*m[8] + a.y*m[9] + a.z*m[10] + a.w*m[11];
}

/* calculates a rotation matrix around X */
void rotation_matrix_X(GLdouble angle, GLdouble m[16])
{
	if (angle != 0.0)
	{
		GLdouble cos_a = cos(angle*M_PI/180.0);
		GLdouble sin_a = sin(angle*M_PI/180.0);

		m[0] = 1.0;
		m[1] = 0.0;
		m[2] = 0.0;
		m[3] = 0.0;

		m[4] = 0.0;
		m[5] = cos_a;
		m[6] = -sin_a;
		m[7] = 0.0;

		m[8] = 0.0;
		m[9] = sin_a;
		m[10] = cos_a;
		m[11] = 0.0;

		m[12] = 0.0;
		m[13] = 0.0;
		m[14] = 0.0;
		m[15] = 1.0;
	}
}

/* calculates a rotation matrix around Y */
void rotation_matrix_Y(GLdouble angle, GLdouble m[16])
{
	if (angle != 0)
	{
		GLdouble cos_a = cos(angle*M_PI/180.0);
		GLdouble sin_a = sin(angle*M_PI/180.0);

		m[0] = cos_a;
		m[1] = 0.0;
		m[2] = sin_a;
		m[3] = 0.0;

		m[4] = 0.0;
		m[5] = 1.0;
		m[6] = 0.0;
		m[7] = 0.0;

		m[8] = -sin_a;
		m[9] = 0.0;
		m[10] = cos_a;
		m[11] = 0.0;

		m[12] = 0.0;
		m[13] = 0.0;
		m[14] = 0.0;
		m[15] = 1.0;
	}
}

/* calculates a rotation matrix around Z */
void rotation_matrix_Z(GLdouble angle, GLdouble m[16])
{
	if (angle != 0)
	{
		GLdouble cos_a = cos(angle*M_PI/180.0);
		GLdouble sin_a = sin(angle*M_PI/180.0);

		m[0] = cos_a;
		m[1] = -sin_a;
		m[2] = 0.0;
		m[3] = 0.0;

		m[4] = sin_a;
		m[5] = cos_a;
		m[6] = 0.0;
		m[7] = 0.0;

		m[8] = 0.0;
		m[9] = 0.0;
		m[10] = 1.0;
		m[11] = 0.0;

		m[12] = 0.0;
		m[13] = 0.0;
		m[14] = 0.0;
		m[15] = 1.0;
	}
}

/* 
 * finds the rotation matrix needed to rotate angle degrees
 * around vector v and stores it in matrix m
 */
void rotation_matrix(GLdouble angle, vector v, GLdouble m[16])
{
	GLdouble x = 0.0, y = 0.0, z = 0.0;	/* components of vector v */
	GLdouble len = 0.0;	/* length of the vector v */
	GLdouble cos_a = 0.0, sin_a = 0.0;

	if (angle != 0.0)
	{
		/* calculate the normalized vector v and store its components in x,y,z */

		len = sqrt(v.x*v.x + v.y*v.y + v.z*v.z);
		if (len != 0.0)
		{
			x = v.x/len;
			y = v.y/len;
			z = v.z/len;
		}

		/* pre-compute the cos(angle), sin(angle) */

		cos_a = cos(angle*M_PI/180.0);
		sin_a = sin(angle*M_PI/180.0);

		/* calculate the elements of the matrix */
		/* first row */

		m[0] = x*x + cos_a*(1 - x*x);
		m[1] = x*y - cos_a*x*y - z*sin_a;
		m[2] = x*z - cos_a*x*z + y*sin_a;
		m[3] = 0.0;

		/* second row */

		m[4] = y*x - cos_a*y*x + z*sin_a;
		m[5] = y*y + cos_a*(1 - y*y);
		m[6] = y*z - cos_a*y*z - x*sin_a;
		m[7] = 0.0;

		/* third row */

		m[8]  = z*x - cos_a*z*x - y*sin_a;
		m[9]  = z*y - cos_a*z*y + x*sin_a;
		m[10] = z*z + cos_a*(1 - z*z);
		m[11] = 0.0;

		/* fourth row */

		m[12] = 0.0;
		m[13] = 0.0;
		m[14] = 0.0;
		m[15] = 1.0;
	}
}

/* finds the translation matrix for the given dx, dy, dz and stores it in m */
void translation_matrix(GLdouble dx, GLdouble dy, GLdouble dz, GLdouble m[16])
{
	m[0] = 1.0;
	m[1] = 0.0;
	m[2] = 0.0;
	m[3] = dx;

	m[4] = 0.0;
	m[5] = 1.0;
	m[6] = 0.0;
	m[7] = dy;

	m[8] = 0.0;
	m[9] = 0.0;
	m[10] = 1.0;
	m[11] = dz;

	m[12] = 0.0;
	m[13] = 0.0;
	m[14] = 0.0;
	m[15] = 1.0;
}

/* prints vector components */
void print_vector(vector x, const char *name)
{
	fprintf(stdout, "Vector %s has components: %f %f %f\n", name, x.x, x.y, x.z);
}

/* prints 4x4 matrix elements */
void print_matrix(GLdouble m[16])
{
	int i;
	
	for (i = 0; i < 16; i++)
		fprintf(stdout, "Matrix element %d equals %f\n", i, m[i]);
}

/* orthonormalizes a matrix, making sure constraints are met */
void orthonormalize_matrix(GLdouble m[16])
{
	vector up, view, right, tmp;

	view.x = m[8];
	view.y = m[9];
	view.z = m[10];
	view.w = 1.0;

	up.x = m[4];
	up.y = m[5];
	up.z = m[6];
	up.w = 1.0;

	view = normalize(view);
	tmp = cross_product(view, up);
	right.x = -tmp.x;
	right.y = -tmp.y;
	right.z = -tmp.z;
	right.w = 1.0;
	right = normalize(right);
	up = cross_product(view, right);

	m[0] = right.x;
	m[1] = right.y;
	m[2] = right.z;

	m[4] = up.x;
	m[5] = up.y;
	m[6] = up.z;

	m[8] = view.x;
	m[9] = view.y;
	m[10] = view.z;
}

/* extract Euler angles from a rotation matrix */
void matrix_to_angles(GLdouble m[16], GLdouble *angleX, GLdouble *angleY, GLdouble *angleZ)
{
	GLdouble tmp = 0.0, tmpx = 0.0, tmpy = 0.0;
	
	*angleY = asin(m[2]);
	tmp = cos(*angleY);
	*angleY = ((*angleY)*180.0)/M_PI;
	// check for gimbal lock
	if (fabs(tmp) > 0.005)
	{
		tmpx = m[10]/tmp;
		tmpy = -m[6]/tmp;
		*angleX = 180.0*atan2(tmpy, tmpx)/M_PI;
		tmpx = m[0]/tmp;
		tmpy = -m[1]/tmp;
		*angleZ = 180.0*atan2(tmpy, tmpx)/M_PI;
	}
	else
	{
		*angleX = 0.0;
		tmpx = m[5];
		tmpy = m[4];
		*angleZ = 180.0*atan2(tmpy, tmpx)/M_PI;
	}
}

// vector operations

/* copies a source to a destination vector */
void copy_vector(vector s, vector *d)
{
	d->x = s.x;
	d->y = s.y;
	d->z = s.z;
	d->w = s.w;
}

/* Returns 1 if two vectors are equal */
GLuint vec_equal(vector u, vector v)
{
	if ((u.x == v.x) && (u.y == v.y) && (u.z == v.z) && (u.w == v.w))
		return 1;
	return 0;
}

/* Returns a difference vector given two vectors */
vector vec_subtract(vector v, vector u)
{
	vector tmp;

	tmp.x = v.x - u.x;
	tmp.y = v.y - u.y;
	tmp.z = v.z - u.z;
	tmp.w = 1.0;

	return(tmp);
}

/* Returns a sum vector given two vectors */
vector vec_add(vector v, vector u)
{
	vector tmp;
	
	tmp.x = v.x + u.x;
	tmp.y = v.y + u.y;
	tmp.z = v.z + u.z;
	tmp.w = 1.0;

	return(tmp);
}

/* Divides a vector by a scalar and returns it */
vector vec_divide_scalar(vector v, GLdouble scalar)
{
	vector tmp;

	tmp.x = v.x / scalar;
	tmp.y = v.y / scalar;
	tmp.z = v.z / scalar;
	tmp.w = 1.0;

	return(tmp);
}

/* Multiplies a vector by a scalar and returns it */
vector vec_multiply_scalar(vector v, GLdouble scalar)
{
	vector tmp;
	
	tmp.x = v.x * scalar;
	tmp.y = v.y * scalar;
	tmp.z = v.z * scalar;
	tmp.w = 1.0;

	return(tmp);
}

/* Returns the negation of the passed vector */
vector vec_negate(vector v)
{
	vector tmp;

	tmp.x = -v.x;
	tmp.y = -v.y;
	tmp.z = -v.z;
	tmp.w = 1.0;

	return(tmp);
}

/* Returns the magnitude of a vector */
GLdouble magnitude(vector v)
{
	return (GLdouble)sqrt((v.x*v.x) + (v.y*v.y) + (v.z*v.z));
}

/* Returns the given vector after normalizing it */
vector normalize(vector v)
{
	GLdouble mag = magnitude(v);

	if (mag != 0.0)
		v = vec_divide_scalar(v, mag);

	return v;
}

/* Returns the dot product between two vectors */
GLdouble dot_product(vector v, vector u)
{
	return((v.x*u.x) + (v.y*u.y) + (v.z*u.z));
}

/* Returns the cross product between two vectors (a perpendicular vector) */
vector cross_product(vector v, vector u)
{
	vector norm;

	norm.x = ((v.y * u.z) - (v.z * u.y));
	norm.y = ((v.z * u.x) - (v.x * u.z));
	norm.z = ((v.x * u.y) - (v.y * u.x));
	norm.w = 1.0;

	return(norm);
}

/* Returns the normal for a polygon given three vertices (vectors) */
vector normal(vector v1, vector v2, vector v3)
{
	vector tmp, norm, edge1, edge2;
	
	edge1 = vec_subtract(v3, v1);
	edge2 = vec_subtract(v2, v1);

	tmp = cross_product(edge1, edge2);
	norm = normalize(tmp);

	return(norm);
}

/* Returns the angle between two vectors */
GLdouble angle_between_vectors(vector v, vector u)
{
	GLdouble dp = 0.0, vm = 0.0;
	GLdouble angle = 0.0;
	
	dp = dot_product(v, u);
	vm = magnitude(v) * magnitude(u);
	angle = (GLdouble)acos(dp/vm);
	if (isnan(angle))
		return 0.0;

	// return the angle in degrees
	//return((180.0 * angle / M_PI));
	// return the angle in radians
	return(angle);
}

/* Returns the distance between two points (vectors) */
GLdouble distance(vector v, vector u)
{
	GLdouble dist = 0.0;
	dist = sqrt((u.x-v.x)*(u.x-v.x) + (u.y-v.y)*(u.y-v.y) + (u.z-v.z)*(u.z-v.z));
	return((GLdouble)dist);
}

/* Rotates vectors axis2 and axis3 around vector axis1 by angle degrees */
void rotate(GLdouble angle, vector axis1, vector *axis2, vector *axis3)
{
	GLdouble m[16];

	if (angle != 0.0)
	{
		rotation_matrix(angle, axis1, m);
		apply_v2m(axis2, m);
		apply_v2m(axis3, m);
	}	
}

/* Translates vector v by dx, dy, and dz */
void translate(GLdouble dx, GLdouble dy, GLdouble dz, vector *v)
{
	if (dx != 0.0)
	{
		v->x += dx * cam->tot[0];
		v->y += dx * cam->tot[1];
		v->z += dx * cam->tot[2];
	}
	if (dy != 0.0)
	{
		v->x += dy * cam->tot[4];
		v->y += dy * cam->tot[5];
		v->z += dy * cam->tot[6];
	}
	if (dz != 0.0)
	{
		v->x += dz * cam->tot[8];
		v->y += dz * cam->tot[9];
		v->z += dz * cam->tot[10];
	}
}

// point and planes operations
/* Returns a point (vector) on the segment u-v given t */
vector point_on_curve(vector u, vector v, GLdouble t)
{
	vector tmp;

	tmp.x = (1.0 - t)*u.x + t*v.x;
	tmp.y = (1.0 - t)*u.y + t*v.y;
	tmp.z = (1.0 - t)*u.z + t*v.z;
	tmp.w = 1.0;

	return tmp;
}

/* Returns a point (vector) on the 4 CP bezier curve according to t and the equation
 * B(t) = P1*(1-t)^3 + P2*3*t*(1-t)^2 + P3*3*t^2*(1-t) + P4*t^3 */
vector point_on_curve2(vector u, vector v, vector w, vector y, GLdouble t)
{
	vector tmp;
	GLdouble term1 = 1.0 - t;
	GLdouble term2 = term1 * term1 * term1;
	GLdouble term3 = t * t * t;
	GLdouble term4 = term1 * term1;
	GLdouble term5 = t * t;

	tmp.x = term2*u.x + 3*t*term4*v.x + 3*term5*term1*w.x + term3*y.x;
	tmp.y = term2*u.y + 3*t*term4*v.y + 3*term5*term1*w.y + term3*y.y;
	tmp.z = term2*u.z + 3*t*term4*v.z + 3*term5*term1*w.z + term3*y.z;
	tmp.w = 1.0;

	return(tmp);
}

/* Computes points on a degree 1 bezier curve using De Casteljau's algorithm */
vector de_cast2(vector v1, vector v2, GLdouble t)
{
	int i, k;
	vector q[2], tmp, tmp1;

	copy_vector(v1, &q[0]);
	copy_vector(v2, &q[1]);

	for (k = 1; k < 2; k++)
	{
		for (i = 0; i < 2 -k; i++)
		{
			tmp = vec_multiply_scalar(q[i], (1.0 - t));
			tmp1 = vec_multiply_scalar(q[i+1], t);
			q[i] = vec_add(tmp, tmp1);
		}
	}
	return (q[0]);
}

/* Computes points on a degree 3 bezier curve using De Casteljau's algorithm */
vector de_cast4(vector v1, vector v2, vector v3, vector v4, GLdouble t)
{
	int i, k;
	vector q[4], tmp, tmp1;

	copy_vector(v1, &q[0]);
	copy_vector(v2, &q[1]);
	copy_vector(v3, &q[2]);
	copy_vector(v4, &q[3]);

	for (k = 1; k < 4; k++)
	{
		for (i = 0; i < 4 -k; i++)
		{
			tmp = vec_multiply_scalar(q[i], (1.0 - t));
			tmp1 = vec_multiply_scalar(q[i+1], t);
			q[i] = vec_add(tmp, tmp1);
		}
	}
	return (q[0]);
}

/* Returns the point (vector) on v1-v2 that is closest to p */
vector closest_point_on_line(vector v1, vector v2, vector p)
{
	vector a, b, c, point, tmp;
	GLdouble d = 0.0, t = 0.0;
	
	a = vec_subtract(p, v1);
	tmp = vec_subtract(v2, v1);
	b = normalize(tmp);
	d = distance(a, b);
	t = dot_product(b, a);
	
	if (t <= 0.0)
		return a;
	if (t >= d)
		return b;
	c = vec_multiply_scalar(b, t);
	point = vec_add(v1, c);
	return(point);
}

/* Returns the distance between a plane and the origin given a normal */
GLdouble plane_distance(vector n, vector p)
{
	GLdouble tmp = 0.0;

	// D = -(Ax + By + Cz)
	tmp = -dot_product(n, p);
	return(tmp);
}

/* Returns 1 if line p1-p2 intersects plane v1-v2-v3 */
GLint intersected_plane(vector v1, vector v2, vector v3, vector p1, vector p2, vector *norm, GLdouble *dist)
{
	GLdouble d1 = 0.0, d2 = 0.0;
	*norm = normal(v1, v2, v3);
	*dist = plane_distance(*norm, v1);
	
	// distance between p1 and the plane
	d1 = dot_product(*norm, p1) + *dist;
	// distance between p2 and the plane
	d2 = dot_product(*norm, p2) + *dist;

	if ((d1*d2) >= 0)
		return 0;
	else
		return 1;
}

/* Returns the point (vector) of intersection between line p1-p2 and a plane */
vector intersection_point(vector norm, vector p1, vector p2, GLdouble dist)
{
	vector line, point, tmp;
	GLdouble num = 0.0, den = 0.0, d = 0.0;

	// get p1-p2 vector
	tmp = vec_subtract(p2, p1);
	line = normalize(tmp);

	// get distance from a point to the plane
	num = -(dot_product(norm, p1) + dist);
	
	den = dot_product(norm, line);
	if (den == 0.0)
		return p1;

	d = (GLdouble)(num/den);
	point.x = (GLdouble)(p1.x + (line.x * d));
	point.y = (GLdouble)(p1.y + (line.y * d));
	point.z = (GLdouble)(p1.z + (line.z * d));
	point.w = 1.0;
	
	return(point);
}

/* Returns 1 if point (vector) is inside a polygon */
GLint inside_polygon(vector intersect, vector poly[], long v_count)
{
	// for rounding error in floating point
	const GLdouble MATCH_FACTOR = 0.99;
	GLdouble angle = 0.0;
	vector a, b;
	int i;
	
	// find vectors between intersection point and every vertex, 2 by 2
	// add up the angles between each couple of vectors
	for (i = 0; i < v_count; i++)
	{
		a = vec_subtract(poly[i], intersect);
		b = vec_subtract(poly[(i+1)%v_count], intersect);
		angle += angle_between_vectors(a, b);
	}

	// if angle is >= 360 degrees the point in inside the polygon
	if ((angle >= (GLdouble)(MATCH_FACTOR*(2.0*M_PI))))
		return 1;
	return 0;
}

/* Rotate vector according to angle around axis */
vector find_rotated_coords(vector vec, GLdouble angle, GLdouble x, GLdouble y, GLdouble z)
{
	if (angle != 0)
	{
		vector new_vec;
		GLdouble vx = vec.x;
		GLdouble vy = vec.y;
		GLdouble vz = vec.z;

		GLdouble cos_theta = (GLdouble)cos(angle);
		GLdouble sin_theta = (GLdouble)sin(angle);

		// find new coordinates for rotated point
		new_vec.x = (cos_theta + (1 - cos_theta) * x * x) * vx;
		new_vec.x += ((1 - cos_theta) * x * y - z * sin_theta) * vy;
		new_vec.x += ((1 - cos_theta) * x * z + y * sin_theta) * vz;

		new_vec.y = ((1 - cos_theta) * x * y + z * sin_theta) * vx;
		new_vec.y += (cos_theta + (1 - cos_theta) * y * y) * vy;
		new_vec.y += ((1 - cos_theta) * y * z - x * sin_theta) * vz;

		new_vec.z  = ((1 - cos_theta) * x * z - y * sin_theta) * vx;
		new_vec.z += ((1 - cos_theta) * y * z + x * sin_theta) * vy;
		new_vec.z += (cos_theta + (1 - cos_theta) * z * z) * vz;

		new_vec.w = 1.0;

		return(new_vec);
	}
	return(vec);
}
