/*------------------------------------------------------------------------------
* Alpha Shooter
* A 3D OpenGL First-Person Shooter game
* Copyright (C) 2005, 2006, 2007 Nicola Cocchiaro <ncocchiaro@users.sf.net>
* Alpha Shooter can be found at http://www.sf.net/projects/alphashooter.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 or, (at your option) any later
* version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*-------------------------------------------------------------------------------
*/

// world.c
// world manipulation functions

#include "world.h"

/* Adds an object structure to the drawable objects list */
object_t* add_object()
{
	object_t* new_obj = NULL;
	
	new_obj = (object_t*)malloc(sizeof(object_t));
	if (new_obj == NULL)
	{
		perror("Unable to allocate memory for new object list item");
		exit(1);
	}
	insert_object(new_obj);
	memset(new_obj, 0, sizeof(object_t));
	return(new_obj);
}

/* Inserts an object in drawable object list */
void insert_object(object_t* o)
{	
	if (o)
	{
		// if object list was empty
		if (obj_list == NULL)
		{
			obj_list = o;
			last_obj = obj_list;
			obj_list->next = NULL;
			obj_list->prev = NULL;
		}
		else
		{
			o->prev = last_obj;
			o->next = NULL;
			last_obj->next = o;
			last_obj = o;
		}
	}
	else
		printf("Attempt to insert a NULL object into drawable list\n");
}

/* Deletes an object from drawable object list */
void delete_object(object_t* o)
{
	object_t *p = NULL, *n = NULL;

	if (o != NULL)
	{
		p = o->prev;
		n = o->next;
		if (p != NULL)
			p->next = n;
		if (n != NULL)
			n->prev = p;
		if (last_obj == o)
			last_obj = p;
		if (obj_list == o)
			obj_list = n;
		free(o);
	}
}

/* Creates an object and sets its structure fields */
object_t *make_object(GLdouble r, GLdouble rotx, GLdouble roty, GLdouble rotz,
GLdouble tx, GLdouble ty, GLdouble tz, void (*drawfun)(void), void (*collidefun)(object_t*))
{
	object_t *tmp = NULL;

	tmp = add_object();
	tmp->radius = r;
	tmp->rotate_around_x = rotx;
	tmp->rotate_around_y = roty;
	tmp->rotate_around_z = rotz;
	tmp->translate_to.x = tx;
	tmp->translate_to.y = ty;
	tmp->translate_to.z = tz;
	copy_vector(tmp->translate_to, &tmp->vp);
	tmp->draw = drawfun;
	tmp->on_collision = collidefun;
	tmp->alpha = 1.0;
	tmp->live = 1;
	return(tmp);
}

/* Adds to the overall score */
void add_score(GLint num)
{
	GLint score = (GLint)atoi(state->score) + num;
	
	sprintf(state->score, "%d", (int)score);
}
